import argparse
import mosamatic2.constants as constants
from flask import Flask, request
from mosamatic2.core.tasks import RescaleDicomImagesTask
from mosamatic2.core.tasks import SegmentMuscleFatL3TensorFlowTask
from mosamatic2.core.tasks import CalculateScoresTask
from mosamatic2.core.tasks import CreatePngsFromSegmentationsTask
from mosamatic2.core.tasks import Dicom2NiftiTask
from mosamatic2.core.tasks import SelectSliceFromScansTask
from mosamatic2.core.tasks import CreateDicomSummaryTask

app = Flask(__name__)


@app.route('/test')
def run_tests():
    return 'PASSED'


@app.route('/rescaledicomimages')
def run_rescaledicomimages():
    images = request.args.get('images')
    target_size = request.args.get('target_size', default=512, type=int)
    output = request.args.get('output')
    overwrite = request.args.get('overwrite', default=True, type=bool)
    task = RescaleDicomImagesTask(
        inputs={'images': images},
        params={'target_size': target_size},
        output=output,
        overwrite=overwrite,
    )
    task.run()
    return 'PASSED'


@app.route('/segmentmusclefatl3tensorflow')
def run_segmentmusclefatl3tensorflow():
    images = request.args.get('images')
    model_files = request.args.get('model_files')
    output = request.args.get('output')
    overwrite = request.args.get('overwrite', default=True, type=bool)
    task = SegmentMuscleFatL3TensorFlowTask(
        inputs={
            'images': images,
            'model_files': model_files,
        },
        params={'model_version': 1.0},
        output=output,
        overwrite=overwrite,
    )
    task.run()
    return 'PASSED'


@app.route('/calculatescores')
def run_calculatescores():
    images = request.args.get('images')
    segmentations = request.args.get('segmentations')
    file_type = request.args.get('file_type', default='npy', type=str)
    output = request.args.get('output')
    overwrite = request.args.get('overwrite', default=True, type=bool)
    task = CalculateScoresTask(
        inputs={
            'images': images,
            'segmentations': segmentations,
        },
        params={'file_type': file_type},
        output=output,
        overwrite=overwrite,
    )
    task.run()
    return 'PASSED'


@app.route('/createpngsfromsegmentations')
def run_createpngsfromsegmentations():
    segmentations = request.args.get('segmentations')
    fig_width = request.args.get('fig_width', default=10, type=int)
    fig_height = request.args.get('fig_height', default=10, type=int)
    output = request.args.get('output')
    overwrite = request.args.get('overwrite', default=True, type=bool)
    task = CreatePngsFromSegmentationsTask(
        inputs={'segmentations': segmentations},
        params={
            'fig_width': fig_width,
            'fig_height': fig_height,
        },
        output=output,
        overwrite=overwrite,
    )
    task.run()
    return 'PASSED'


@app.route('/dicom2nifti')
def run_dicom2nifti():
    images = request.args.get('images')
    output = request.args.get('output')
    overwrite = request.args.get('overwrite', default=True, type=bool)
    task = Dicom2NiftiTask(
        inputs={'images': images},
        params=None,
        output=output,
        overwrite=overwrite,
    )
    task.run()
    return 'PASSED'


@app.route('/selectslicefromscans')
def run_selectslicefromscans():
    scans = request.args.get('scans')
    vertebra = request.args.get('vertebra')
    output = request.args.get('output')
    overwrite = request.args.get('overwrite', default=True, type=bool)
    task = SelectSliceFromScansTask(
        inputs={'scans': scans},
        params={'vertebra': vertebra},
        output=output,
        overwrite=overwrite,
    )
    task.run()
    return 'PASSED'


@app.route('/createdicomsummary')
def run_createdicomsummary():
    directory = request.args.get('directory')
    output = request.args.get('output')
    overwrite = request.args.get('overwrite', default=True, type=bool)
    task = CreateDicomSummaryTask(
        inputs={'directory': directory},
        params=None,
        output=output,
        overwrite=overwrite,
    )
    task.run()
    return 'PASSED'


def main():
    parser = argparse.ArgumentParser()
    parser.add_argument('--port', type=int, default=constants.MOSAMATIC2_SERVER_PORT)
    parser.add_argument('--debug', type=bool, default=constants.MOSAMATIC2_SERVER_DEBUG)
    args = parser.parse_args()
    app.run(host='0.0.0.0', port=args.port, debug=args.debug)


import sys
from PySide6.QtWidgets import QApplication, QMainWindow, QVBoxLayout, QWidget
import pyqtgraph as pg
import numpy as np


class MainWindow(QMainWindow):
    def __init__(self):
        super().__init__()
        self.setWindowTitle("PySide6 + PyQtGraph Example")
        self.resize(600, 400)

        # Central widget
        central = QWidget()
        layout = QVBoxLayout(central)
        self.setCentralWidget(central)

        # Data
        categories = ["Apples", "Bananas", "Cherries", "Dates", "Elderberries"]
        values = np.array([12, 23, 36, 18, 29])

        # Bar chart
        bar = pg.BarGraphItem(
            x=np.arange(len(categories)),  # x positions
            height=values,
            width=0.6,
            brush="skyblue"
        )

        plot = pg.PlotWidget()
        plot.addItem(bar)

        # Customize look
        plot.setBackground("w")
        plot.setTitle("Fruit Counts", color="k", size="14pt")
        plot.showGrid(x=True, y=True)
        plot.setYRange(0, max(values) + 5)

        # Set x-axis ticks to category names
        axis = plot.getAxis("bottom")
        axis.setTicks([list(enumerate(categories))])

        layout.addWidget(plot)


if __name__ == "__main__":
    app = QApplication(sys.argv)
    win = MainWindow()
    win.show()
    sys.exit(app.exec())



import click

from mosamatic2.core.tasks import CalculateMaskStatisticsTask


@click.command(help='Calculates segmentation mask statistics')
@click.option(
    '--scans', 
    required=True, 
    type=click.Path(exists=True), 
    help='Directory with scans in NIFTI format',
)
@click.option(
    '--masks',
    required=True,
    type=click.Path(exists=True), 
    help='Directory with segmentation mask files in NIFTI format',
)
@click.option(
    '--output', 
    required=True, 
    type=click.Path(), 
    help='Output directory'
)
@click.option(
    '--overwrite', 
    type=click.BOOL, 
    default=False, 
    help='Overwrite [true|false]'
)
def calculatemaskstatistics(scans, masks, output, overwrite):
    """
    Calculates segmentation mask statistics. The following metrics are calculated:

    (1) Mean radiation attenuation (HU)
    (2) Standard deviation radiation attenuation (HU)
    (3) Segmentation mask volume (mL)
    
    Parameters
    ----------
    --scans : str
        Directory with scans in NIFTI format

    --masks : str
        Directory with segmentation mask files in NIFTI format

    --output : str
        Path to output directory

    --overwrite : bool
        Overwrite contents output directory [true|false]
    """
    task = CalculateMaskStatisticsTask(
        inputs={'scans': scans, 'masks': masks},
        params=None,
        output=output,
        overwrite=overwrite,
    )
    task.run()


import os

from PySide6.QtWidgets import (
    QLineEdit,
    QSpinBox,
    QComboBox,
    QCheckBox,
    QHBoxLayout,
    QVBoxLayout,
    QFormLayout,
    QPushButton,
    QFileDialog,
    QMessageBox,
)
from PySide6.QtCore import (
    QThread, 
    Slot,
)

from mosamatic2.core.managers.logmanager import LogManager
from mosamatic2.ui.widgets.panels.pipelines.pipelinepanel import PipelinePanel
from mosamatic2.ui.settings import Settings
from mosamatic2.ui.utils import is_macos
from mosamatic2.ui.worker import Worker
from mosamatic2.core.pipelines import DefaultPipeline

LOG = LogManager()

PANEL_TITLE = 'DefaultPipeline'
PANEL_NAME = 'defaultpipelinepanel'
MODEL_TYPE_ITEM_NAMES = ['tensorflow', 'pytorch']
MODEL_VERSION_ITEM_NAMES = ['1.0', '2.2']


class DefaultPipelinePanel(PipelinePanel):
    def __init__(self):
        super(DefaultPipelinePanel, self).__init__()
        self.set_title(PANEL_TITLE)
        self._images_dir_line_edit = None
        self._images_dir_select_button = None
        self._model_files_dir_line_edit = None
        self._model_files_dir_select_button = None
        self._output_dir_line_edit = None
        self._output_dir_select_button = None
        self._target_size_spinbox = None
        self._model_type_combobox = None
        self._model_version_combobox = None
        self._fig_width_spinbox = None
        self._fig_height_spinbox = None
        self._full_scan_checkbox = None
        self._overwrite_checkbox = None
        self._form_layout = None
        self._run_pipeline_button = None
        self._settings = None
        self._task = None
        self._worker = None
        self._thread = None
        self.init_layout()

    def images_dir_line_edit(self):
        if not self._images_dir_line_edit:
            self._images_dir_line_edit = QLineEdit()
            self._images_dir_line_edit.setText(self.settings().get(f'{PANEL_NAME}/images_dir'))
        return self._images_dir_line_edit
    
    def images_dir_select_button(self):
        if not self._images_dir_select_button:
            self._images_dir_select_button = QPushButton('Select')
            self._images_dir_select_button.clicked.connect(self.handle_images_dir_select_button)
        return self._images_dir_select_button
    
    def model_files_dir_line_edit(self):
        if not self._model_files_dir_line_edit:
            self._model_files_dir_line_edit = QLineEdit()
            self._model_files_dir_line_edit.setText(self.settings().get(f'{PANEL_NAME}/model_files_dir'))
        return self._model_files_dir_line_edit
    
    def model_files_dir_select_button(self):
        if not self._model_files_dir_select_button:
            self._model_files_dir_select_button = QPushButton('Select')
            self._model_files_dir_select_button.clicked.connect(self.handle_model_files_dir_select_button)
        return self._model_files_dir_select_button
    
    def output_dir_line_edit(self):
        if not self._output_dir_line_edit:
            self._output_dir_line_edit = QLineEdit()
            self._output_dir_line_edit.setText(self.settings().get(f'{PANEL_NAME}/output_dir'))
        return self._output_dir_line_edit
    
    def output_dir_select_button(self):
        if not self._output_dir_select_button:
            self._output_dir_select_button = QPushButton('Select')
            self._output_dir_select_button.clicked.connect(self.handle_output_dir_select_button)
        return self._output_dir_select_button
    
    def target_size_spinbox(self):
        if not self._target_size_spinbox:
            self._target_size_spinbox = QSpinBox()
            self._target_size_spinbox.setMinimum(0)
            self._target_size_spinbox.setMaximum(1024)
            self._target_size_spinbox.setValue(self.settings().get_int(f'{PANEL_NAME}/target_size', 512))
        return self._target_size_spinbox
    
    def model_type_combobox(self):
        if not self._model_type_combobox:
            self._model_type_combobox = QComboBox()
            self._model_type_combobox.addItems(MODEL_TYPE_ITEM_NAMES)
            self._model_type_combobox.setCurrentText(self.settings().get(f'{PANEL_NAME}/model_type'))
            self._model_type_combobox.currentTextChanged.connect(self.handle_model_type_combobox)
        return self._model_type_combobox
    
    def model_version_combobox(self):
        if not self._model_version_combobox:
            self._model_version_combobox = QComboBox()
            self._model_version_combobox.addItems(MODEL_VERSION_ITEM_NAMES)
            self._model_version_combobox.setCurrentText(self.settings().get(f'{PANEL_NAME}/model_version'))
            self._model_version_combobox.currentTextChanged.connect(self.handle_model_version_combobox)
        return self._model_version_combobox
    
    def fig_width_spinbox(self):
        if not self._fig_width_spinbox:
            self._fig_width_spinbox = QSpinBox()
            self._fig_width_spinbox.setValue(self.settings().get_int(f'{PANEL_NAME}/fig_width', default=10))
        return self._fig_width_spinbox
    
    def fig_height_spinbox(self):
        if not self._fig_height_spinbox:
            self._fig_height_spinbox = QSpinBox()
            self._fig_height_spinbox.setValue(self.settings().get_int(f'{PANEL_NAME}/fig_height', default=10))
        return self._fig_height_spinbox
    
    def full_scan_checkbox(self):
        if not self._full_scan_checkbox:
            self._full_scan_checkbox = QCheckBox('')
            self._full_scan_checkbox.setChecked(self.settings().get_bool(f'{PANEL_NAME}/full_scan', False))
        return self._full_scan_checkbox

    def overwrite_checkbox(self):
        if not self._overwrite_checkbox:
            self._overwrite_checkbox = QCheckBox('')
            self._overwrite_checkbox.setChecked(self.settings().get_bool(f'{PANEL_NAME}/overwrite', True))
        return self._overwrite_checkbox
    
    def form_layout(self):
        if not self._form_layout:
            self._form_layout = QFormLayout()
            if is_macos():
                self._form_layout.setFieldGrowthPolicy(QFormLayout.ExpandingFieldsGrow)
        return self._form_layout
    
    def run_pipeline_button(self):
        if not self._run_pipeline_button:
            self._run_pipeline_button = QPushButton('Run pipeline')
            self._run_pipeline_button.clicked.connect(self.handle_run_pipeline_button)
        return self._run_pipeline_button
    
    def settings(self):
        if not self._settings:
            self._settings = Settings()
        return self._settings
    
    def init_help_dialog(self):
        self.help_dialog().set_text('Show some help information')
    
    def init_layout(self):
        images_dir_layout = QHBoxLayout()
        images_dir_layout.addWidget(self.images_dir_line_edit())
        images_dir_layout.addWidget(self.images_dir_select_button())
        model_files_dir_layout = QHBoxLayout()
        model_files_dir_layout.addWidget(self.model_files_dir_line_edit())
        model_files_dir_layout.addWidget(self.model_files_dir_select_button())
        output_dir_layout = QHBoxLayout()
        output_dir_layout.addWidget(self.output_dir_line_edit())
        output_dir_layout.addWidget(self.output_dir_select_button())
        self.form_layout().addRow('Images directory', images_dir_layout)
        self.form_layout().addRow('Model files directory', model_files_dir_layout)
        self.form_layout().addRow('Output directory', output_dir_layout)
        self.form_layout().addRow('Overwrite', self.overwrite_checkbox())
        layout = QVBoxLayout()
        layout.addLayout(self.form_layout())
        layout.addWidget(self.run_pipeline_button())
        self.setLayout(layout)
        self.setObjectName(PANEL_NAME)

    def handle_images_dir_select_button(self):
        last_directory = self.settings().get('last_directory')
        directory = QFileDialog.getExistingDirectory(dir=last_directory)
        if directory:
            self.images_dir_line_edit().setText(directory)
            self.settings().set('last_directory', directory)

    def handle_model_files_dir_select_button(self):
        last_directory = self.settings().get('last_directory')
        directory = QFileDialog.getExistingDirectory(dir=last_directory)
        if directory:
            self.model_files_dir_line_edit().setText(directory)
            self.settings().set('last_directory', directory)

    def handle_output_dir_select_button(self):
        last_directory = self.settings().get('last_directory')
        directory = QFileDialog.getExistingDirectory(dir=last_directory)
        if directory:
            self.output_dir_line_edit().setText(directory)
            self.settings().set('last_directory', directory)

    def handle_model_type_combobox(self, text):
        if text == 'tensorflow':
            self.model_version_combobox().setCurrentText('1.0')
        if text == 'pytorch':
            self.model_version_combobox().setCurrentText('2.2')

    def handle_model_version_combobox(self, text):
        if text == '1.0':
            self.model_type_combobox().setCurrentText('tensorflow')
        if text == '2.2':
            self.model_type_combobox().setCurrentText('pytorch')

    def handle_run_pipeline_button(self):
        errors = self.check_inputs_and_parameters()
        if len(errors) > 0:
            error_message = 'Following errors were encountered:\n'
            for error in errors:
                error_message += f' - {error}\n'
            QMessageBox.information(self, 'Error', error_message)
        else:
            LOG.info('Running pipeline...')
            self.run_pipeline_button().setEnabled(False)
            self.save_inputs_and_parameters()
            self._task = DefaultPipeline(
                inputs={
                    'images': self.images_dir_line_edit().text(),
                    'model_files': self.model_files_dir_line_edit().text(),
                },
                params={
                    'file_type': 'npy',
                    'model_type': 'tensorflow',
                    'model_version': 1.0,
                    'target_size': 512,
                    'fig_width': 10,
                    'fig_height': 10,
                    'hu_low': 30,
                    'hu_high': 150,
                    'alpha': 1.0,
                },
                output=self.output_dir_line_edit().text(),
                overwrite=self.overwrite_checkbox().isChecked(),
            )
            self._worker = Worker(self._task)
            self._thread = QThread()
            self._worker.moveToThread(self._thread)
            self._thread.started.connect(self._worker.run)
            self._worker.progress.connect(self.handle_progress)
            self._worker.status.connect(self.handle_status)
            self._worker.finished.connect(self.handle_finished)
            self._worker.finished.connect(self._thread.quit)
            self._worker.finished.connect(self._worker.deleteLater)
            self._thread.finished.connect(self._thread.deleteLater)
            self._thread.start()

    @Slot(int)
    def handle_progress(self, progress):
        LOG.info(f'Progress: {progress} / 100%')

    @Slot(str)
    def handle_status(self, status):
        LOG.info(f'Status: {status}')

    @Slot()
    def handle_finished(self):
        self.run_pipeline_button().setEnabled(True)

    def check_inputs_and_parameters(self):
        errors = []
        if self.images_dir_line_edit().text() == '':
            errors.append('Empty images directory path')
        elif not os.path.isdir(self.images_dir_line_edit().text()):
            errors.append('Images directory does not exist')
        if self.model_files_dir_line_edit().text() == '':
            errors.append('Empty model files directory path')
        elif not os.path.isdir(self.model_files_dir_line_edit().text()):
            errors.append('Model files directory does not exist')
        if self.output_dir_line_edit().text() == '':
            errors.append('Empty output directory path')
        elif os.path.isdir(self.output_dir_line_edit().text()) and not self.overwrite_checkbox().isChecked():
            errors.append('Output directory exists but overwrite=False. Please remove output directory first')
        # if self.target_size_spinbox().value() != 512:
        #     errors.append('Target size must be 512')
        # if self.full_scan_checkbox().isChecked():
        #     errors.append('Full scan support is not available yet')
        return errors
    
    def save_inputs_and_parameters(self):
        self.settings().set(f'{PANEL_NAME}/images_dir', self.images_dir_line_edit().text())
        self.settings().set(f'{PANEL_NAME}/model_files_dir', self.model_files_dir_line_edit().text())
        self.settings().set(f'{PANEL_NAME}/output_dir', self.output_dir_line_edit().text())
        # self.settings().set(f'{PANEL_NAME}/target_size', self.target_size_spinbox().value())
        # self.settings().set(f'{PANEL_NAME}/model_type', self.model_type_combobox().currentText())
        # self.settings().set(f'{PANEL_NAME}/model_version', self.model_version_combobox().currentText())
        # self.settings().set(f'{PANEL_NAME}/fig_width', self.fig_width_spinbox().value())
        # self.settings().set(f'{PANEL_NAME}/fig_height', self.fig_height_spinbox().value())
        # self.settings().set(f'{PANEL_NAME}/full_scan', self.full_scan_checkbox().isChecked())
        self.settings().set(f'{PANEL_NAME}/overwrite', self.overwrite_checkbox().isChecked())


import click

from mosamatic2.core.tasks import TotalSegmentatorTask


@click.command(help='Run Total Segmentator on CT scans')
@click.option(
    '--scans', 
    required=True, 
    type=click.Path(exists=True), 
    help='Directory with scans (each patient separate subdirectory)',
)
@click.option(
    '--output', 
    required=True, 
    type=click.Path(), 
    help='Output directory'
)
@click.option(
    '--tasks', 
    default='total',
    help='Comma-separated list of Total Segmentator tasks to run (no spaces!)'
)
@click.option(
    '--format',
    default='dicom',
    help='Process scans in DICOM or NIFTI format [dicom|nifti] (default: dicom)'
)
@click.option(
    '--overwrite', 
    type=click.BOOL, 
    default=False, 
    help='Overwrite [true|false]'
)
def totalsegmentator(scans, tasks, format, output, overwrite):
    """
    Run Total Segmentator on CT scans. If you want to run specialized tasks
    like "liver_segments" or "liver_vessels" you need an educational license.
    Check out https://github.com/wasserth/TotalSegmentator?tab=readme-ov-file
    to find out how to get such a license.
    
    Parameters
    ----------
    --scans : str
        Directory to scans. Each patient's scan should be in a separate
        subdirectory. For example:

        /scans
        /scans/patient1
        /scans/patient1/file1.dcm
        /scans/patient1/file2.dcm
        ...
        /scans/patient2
        ...

    --output : str
        Path to output directory where selected slices will be placed. Each
        slice's file name will be the same as the scan directory name, so in
        the example above that would be "patient1", "patient2", etc.

    --tasks : str
        Comma-separated list of Total Segmentator tasks to run (no spaces!)

    --format : str
        Process scans in DICOM or NIFTI format. Options: [dicom|nifti]
        Default is 'dicom'.

    --overwrite : bool
        Overwrite contents output directory [true|false]
    """
    task = TotalSegmentatorTask(
        inputs={'scans': scans},
        params={'tasks': tasks, 'format': format},
        output=output,
        overwrite=overwrite,
    )
    task.run()


import os
import time
import math
import json
import pydicom
import pydicom.errors
import constants
from pathlib import Path
from webdav3.client import Client


def get_webdav_token():
    token_file_path = 'C:\\Users\\r.brecheisen\\mdr-webdav.txt'
    with open(token_file_path, 'r') as f:
        token = f.readline().strip()
    return token


def get_webdav_client():
    options = {
        'webdav_hostname': "https://download.datahubmaastricht.nl",
        'webdav_login':    "rbrecheise",
        'webdav_password': get_webdav_token(),
    }
    client = Client(options)
    return client


def get_path(file_name):
    return os.path.join(constants.BASE_DIR, file_name)


def get_parent_path(file_path):
    parent = Path(file_path).parent
    return str(parent)


def get_parent_name(file_path):
    return Path(file_path).parent.name


def print_dictionary(dictionary):
    for k, v in dictionary.items():
        print(f'{k}: {v}')


def load_dictionary(file_name, reverse=False):
    dictionary = None
    with open(get_path(file_name), 'r') as f:
        dictionary = json.load(f)
    if reverse:
        return {v: k for k, v in dictionary.items()}
    return dictionary


def save_dictionary(dictionary, file_name):
    with open(get_path(file_name), 'w') as f:
        json.dump(dictionary, f, indent=4)


def is_number(value):
    try:
        int(value)
        return True
    except:
        return False
    

def is_valid_dicom(f_path):
    try:
        p = pydicom.dcmread(f_path, stop_before_pixels=True)
        if 'Modality' in p and p.Modality == 'CT' and 'SeriesInstanceUID' in p:
            return True
        return False
    except pydicom.errors.InvalidDicomError:
        return False
    

def get_series_instance_uid(file_path):
    p = pydicom.dcmread(file_path, stop_before_pixels=True)
    if 'SeriesInstanceUID' in p:
        return p.SeriesInstanceUID
    return None


def current_time_in_milliseconds():
    return int(round(time.time() * 1000))


def current_time_in_seconds() -> int:
    return int(round(current_time_in_milliseconds() / 1000.0))


def elapsed_time_in_milliseconds(start_time_in_milliseconds):
    return current_time_in_milliseconds() - start_time_in_milliseconds


def elapsed_time_in_seconds(start_time_in_seconds):
    return current_time_in_seconds() - start_time_in_seconds


def duration(seconds):
    h = int(math.floor(seconds/3600.0))
    remainder = seconds - h * 3600
    m = int(math.floor(remainder/60.0))
    remainder = remainder - m * 60
    s = int(math.floor(remainder))
    return '{} hours, {} minutes, {} seconds'.format(h, m, s)


import os
import numpy as np
import pandas as pd
from mosamatic2.core.tasks.task import Task
from mosamatic2.core.managers.logmanager import LogManager
from mosamatic2.core.data.multidicomimage import MultiDicomImage
from mosamatic2.core.data.numpyimage import NumpyImage
from mosamatic2.core.utils import (
    get_pixels_from_dicom_object,
    calculate_area,
    calculate_mean_radiation_attenuation,
    calculate_lama_percentage,
    get_pixels_from_tag_file,
    MUSCLE,
    SAT,
    VAT,
)

LOG = LogManager()


class ApplyThresholdToSegmentationsTask(Task):
    INPUTS = [
        'images',
        'segmentations'
    ]
    PARAMS = [
        'label',
        'threshold_low', 
        'threshold_high'
    ]

    def __init__(self, inputs, params, output, overwrite=True):
        super(ApplyThresholdToSegmentationsTask, self).__init__(inputs, params, output, overwrite)

    def collect_image_segmentation_pairs(self, images, segmentations, file_type='npy'):
        file_type = '.tag' if file_type == 'tag' else '.seg.npy'
        img_seg_pairs = []
        for image in images:
            f_img_path = image.path()
            f_img_name = os.path.split(f_img_path)[1]
            for f_seg_path in segmentations:
                f_seg_name = os.path.split(f_seg_path)[1]
                if file_type == '.seg.npy':
                    f_seg_name = f_seg_name.removesuffix(file_type)
                    if f_seg_name == f_img_name:
                        img_seg_pairs.append((image, f_seg_path))
                elif file_type == '.tag':
                    f_seg_name = f_seg_name.removesuffix(file_type).removesuffix('.dcm')
                    f_img_name = f_img_name.removesuffix('.dcm')
                    if f_seg_name == f_img_name:
                        img_seg_pairs.append((image, f_seg_path))
                else:
                    raise RuntimeError('Unknown file type')
        return img_seg_pairs

    def load_images(self):
        image_data = MultiDicomImage()
        image_data.set_path(self.input('images'))
        if image_data.load():
            return image_data
        raise RuntimeError('Could not load images')
    
    def load_pixels_and_spacing(self, image):
        p = image.object()
        pixels = get_pixels_from_dicom_object(p, normalize=True)
        return pixels, p.PixelSpacing

    def load_segmentations(self, file_type='npy'):
        file_type = '.tag' if file_type == 'tag' else '.seg.npy'
        segmentations = []
        for f in os.listdir(self.input('segmentations')):
            f_path = os.path.join(self.input('segmentations'), f)
            if f.endswith(file_type):
                segmentations.append(f_path)
        if len(segmentations) == 0:
            raise RuntimeError('Input directory has no segmentation files')
        return segmentations

    def load_segmentation(self, f, file_type='npy'):
        if file_type == 'npy':
            segmentation = NumpyImage()
            segmentation.set_path(f)
            if segmentation.load():
                return segmentation.object()
            LOG.error(f'Could not load segmentation file {f}')
        if file_type == 'tag':
            pixels = get_pixels_from_tag_file(f)
            try:
                pixels = pixels.reshape(512, 512)
                return pixels
            except Exception:
                LOG.warning(f'Could not reshape TAG pixels to (512, 512), skipping...')
                return None
        raise RuntimeError('Unknown file type')

    def run(self):
        images = self.load_images().images()
        segmentations = self.load_segmentations()
        image_segmentation_pairs = self.collect_image_segmentation_pairs(images, segmentations)
        threshold_low = self.param('threshold_low')
        threshold_high = self.param('threshold_high')
        label = self.param('label')
        nr_steps = len(image_segmentation_pairs)
        for step in range(nr_steps):
            image, pixel_spacing = self.load_pixels_and_spacing(image_segmentation_pairs[step][0])
            segmentation = self.load_segmentation(image_segmentation_pairs[step][1])
            area = calculate_area(segmentation, label, pixel_spacing)
            LOG.info(f'area: {area} mm')
            segmentation_new = segmentation.copy()
            segmentation_new[(segmentation_new == label) & (image < threshold_low)] = 0
            area_new = calculate_area(segmentation_new, label, pixel_spacing)
            LOG.info(f'area_new: {area_new} mm')
            segmentation_new_file_name = os.path.split(image_segmentation_pairs[step][0].path())[1]
            segmentation_new_file_path = os.path.join(self.output(), f'{segmentation_new_file_name}.seg.npy')
            np.save(segmentation_new_file_path, segmentation_new)
            self.set_progress(step, nr_steps)


import os
import shutil
import numpy as np
from mosamatic2.core.tasks.task import Task
from mosamatic2.core.managers.logmanager import LogManager
from mosamatic2.core.data.multidicomimage import MultiDicomImage
from mosamatic2.core.data.dicomimage import DicomImage
from mosamatic2.core.data.multidicomimage import MultiDicomImage
from scipy.ndimage import zoom

LOG = LogManager()


class RescaleDicomImagesTask(Task):
    INPUTS = ['images']
    PARAMS = ['target_size']

    def __init__(self, inputs, params, output=None, overwrite=True):
        super(RescaleDicomImagesTask, self).__init__(inputs, params, output, overwrite)

    def rescale_image(self, p, target_size):
        pixel_array = p.pixel_array
        hu_array = pixel_array * p.RescaleSlope + p.RescaleIntercept
        hu_air = -1000
        new_rows = max(p.Rows, p.Columns)
        new_cols = max(p.Rows, p.Columns)
        padded_hu_array = np.full((new_rows, new_cols), hu_air, dtype=hu_array.dtype)
        padded_hu_array[:pixel_array.shape[0], :pixel_array.shape[1]] = hu_array
        pixel_array_padded = (padded_hu_array - p.RescaleIntercept) / p.RescaleSlope
        pixel_array_padded = pixel_array_padded.astype(pixel_array.dtype) # Image now has largest dimensions
        pixel_array_rescaled = zoom(pixel_array_padded, zoom=(target_size / new_rows), order=3) # Cubic interpolation
        pixel_array_rescaled = pixel_array_rescaled.astype(pixel_array.dtype)
        original_pixel_spacing = p.PixelSpacing
        new_pixel_spacing = [ps * (new_rows / target_size) for ps in original_pixel_spacing]
        p.PixelSpacing = new_pixel_spacing
        p.PixelData = pixel_array_rescaled.tobytes()
        p.Rows = target_size
        p.Columns = target_size
        return p

    def run(self):
        image_data = MultiDicomImage()
        image_data.set_path(self.input('images'))
        if image_data.load():
            images = image_data.images()
            nr_steps = len(images)
            for step in range(nr_steps):
                source = images[step]
                assert isinstance(source, DicomImage)
                p = source.object()
                if len(p.pixel_array.shape) == 2:
                    source_name = os.path.split(source.path())[1]
                    if p.Rows != self.param('target_size') or p.Columns != self.param('target_size'):
                        p = self.rescale_image(p, self.param('target_size'))
                        target = os.path.join(self.output(), source_name)
                        p.save_as(target)
                    else:
                        target = os.path.join(self.output(), source_name)
                        shutil.copy(source.path(), target)
                else:
                    LOG.warning(f'Shape of pixel data in file {source.path()} should be 2D but is {len(p.pixel_array.shape)}D')
                self.set_progress(step, nr_steps)
        else:
            LOG.error('Error loading multi-DICOM image data')


from PySide6.QtWidgets import QWidget, QPushButton

from mosamatic2.ui.widgets.dialogs.helpdialog import HelpDialog
from mosamatic2.ui.settings import Settings


class DefaultPanel(QWidget):
    def __init__(self):
        super(DefaultPanel, self).__init__()
        self._title = None
        self._help_dialog = None
        self._show_help_button = None
        self._settings = None

    def title(self):
        return self._title
    
    def set_title(self, title):
        self._title = title

    def help_dialog(self):
        if not self._help_dialog:
            self._help_dialog = HelpDialog()
        return self._help_dialog

    def settings(self):
        if not self._settings:
            self._settings = Settings()
        return self._settings

    def show_help_button(self):
        if not self._show_help_button:
            self._show_help_button = QPushButton('Help')
            self._show_help_button.clicked.connect(self.handle_show_help_button)
        return self._show_help_button
    
    def handle_show_help_button(self):
        self.help_dialog().show()


import sys
import os
import numpy as np
import matplotlib.pyplot as plt
import pydicom

from matplotlib.widgets import Slider

DICOM_FILE_NAME = '04RAND092.dcm'
SEGMENTATION_FILE_NAME = '04RAND092.dcm.seg.npy'
# File paths for demo Marcel and Bibi
DICOM_FILE_PATH = os.path.join('/Volumes/PHILIPS UFD/marcelvdpoll/original', DICOM_FILE_NAME)
SEGMENTATION_FILE_PATH = os.path.join('/Volumes/PHILIPS UFD/marcelvdpoll/output/defaultpipeline/segmentmusclefatl3tensorflowtask', SEGMENTATION_FILE_NAME)
# DICOM_FILE_PATH = f'/Users/ralph/Library/CloudStorage/GoogleDrive-ralph.brecheisen@gmail.com/My Drive/data/mosamatic2/data/l3_edema/{DICOM_FILE_NAME}'
# if sys.platform.startswith('win'):
#     DICOM_FILE_PATH = f'G:\\My Drive\\data\\mosamatic2\\data\\l3_edema\\{DICOM_FILE_NAME}'
# SEGMENTATION_FILE_PATH = f'/Users/ralph/Library/CloudStorage/GoogleDrive-ralph.brecheisen@gmail.com/My Drive/data/mosamatic2/data/l3_edema/{SEGMENTATION_FILE_NAME}'
# if sys.platform.startswith('win'):
#     SEGMENTATION_FILE_PATH = f'G:\\My Drive\\data\\mosamatic2\\data\\l3_edema\\{SEGMENTATION_FILE_NAME}'

HU_MIN = -29
HU_MAX = 150

slider_lo_ax = None
slider_hi_ax = None
slider_lo = None
slider_hi = None
image = None
muscle_mask = None
hu_lo = 30
hu_hi = 150
fig = None
axes = None
slider_lo_text = None
slider_hi_text = None


def rgba_from_binary_mask(mask, rgba, alpha):
    out = np.zeros((*mask.shape, 4), dtype=np.float32)
    out[mask] = (rgba[0], rgba[1], rgba[2], alpha)
    return out


def rgba_from_binary_mask_thresholded(image, mask, label, hu_lo, hu_hi, alpha):
    mask = (mask == label)
    red = mask & (image >= hu_lo) & (image <= hu_hi)
    yellow = mask & (image < hu_lo)
    overlay = np.zeros((*image.shape, 4), dtype=np.float32)
    overlay[red] = (1.0, 0.0, 0.0, alpha)
    overlay[yellow] = (1.0, 1.0, 0.0, alpha)
    return overlay


def on_change_lo(value):
    axes[1].clear()
    axes[1].imshow(apply_window_and_level(image), cmap="gray")
    axes[1].imshow(rgba_from_binary_mask_thresholded(image, muscle_mask, label=1, hu_lo=value, hu_hi=hu_hi, alpha=1))
    axes[1].axis("off")
    axes[1].set_title(f"Muscle low-RA (yellow) and high-RA (red)")
    fig.canvas.draw_idle()


def on_change_hi(value):
    axes[1].clear()
    axes[1].imshow(apply_window_and_level(image), cmap="gray")
    axes[1].imshow(rgba_from_binary_mask_thresholded(image, muscle_mask, label=1, hu_lo=hu_lo, hu_hi=value, alpha=1))
    axes[1].axis("off")
    axes[1].set_title(f"Muscle low-RA (yellow) and high-RA (red)")
    fig.canvas.draw_idle()


def apply_window_and_level(img, window=400, level=50):
    lo = level - window / 2.0
    hi = level + window / 2.0
    img = np.clip(img, lo, hi)
    img = (img - lo) / (hi - lo)          # -> 0..1
    return (img * 255.0 + 0.5)


def main():
    global fig, axes, image, muscle_mask, slider_lo, slider_lo_ax, slider_lo_text

    # Load image
    p = pydicom.dcmread(DICOM_FILE_PATH)
    image = p.pixel_array.astype(np.float32)
    image = image * float(getattr(p, "RescaleSlope", 1.0)) + float(getattr(p, "RescaleIntercept", 0.0))

    # Load segmentation mask
    mask = np.load(SEGMENTATION_FILE_PATH).astype(np.uint8)
    
    muscle_mask = (mask == 1)
    overlay_muscle = rgba_from_binary_mask(muscle_mask, rgba=(1.0, 0.0, 0.0), alpha=1)
    overlay_myosteatosis = rgba_from_binary_mask_thresholded(image, muscle_mask, label=1, hu_lo=hu_lo, hu_hi=hu_hi, alpha=1)

    # Create plots
    fig, axes = plt.subplots(1, 2, figsize=(14, 7))    
    axes[0].imshow(apply_window_and_level(image), cmap='gray')
    axes[0].imshow(overlay_muscle)
    axes[0].axis("off")
    axes[0].set_title('Muscle (red)')
    axes[1].imshow(apply_window_and_level(image), cmap="gray")
    axes[1].imshow(overlay_myosteatosis)
    axes[1].axis("off")
    axes[1].set_title(f"Muscle low-RA (yellow) and high-RA (red)")

    # Create sliders for low and high thresholds
    slider_lo_ax = fig.add_axes([0.15, 0.06, 0.7, 0.03])
    slider_lo = Slider(slider_lo_ax, "Low-RA", HU_MIN, HU_MAX, valinit=hu_lo)
    slider_lo.on_changed(on_change_lo)
    slider_hi_ax = fig.add_axes([0.15, 0.03, 0.7, 0.03])
    slider_hi = Slider(slider_hi_ax, "High-RA", HU_MIN, HU_MAX, valinit=hu_hi)
    slider_hi.on_changed(on_change_hi)

    plt.show()


if __name__ == "__main__":
    main()


import os
import shutil
from mosamatic2.core import utils


def main():
    source_dir = '/Users/ralph/Desktop/downloads/coco/Archief'
    target_dir = '/Users/ralph/Desktop/downloads/coco/Archief_prep'
    os.makedirs(target_dir, exist_ok=True)
    for root, dirs, files in os.walk(source_dir):
        for f in files:
            f_path = os.path.join(root, f)
            if os.path.isfile(f_path) and utils.is_dicom(f_path) and not f == 'DICOMDIR':
                shutil.copyfile(f_path, os.path.join(target_dir, f + '.dcm'))
                print(f_path)


if __name__ == '__main__':
    main()


import os
import utils
import constants


def main():

    client = utils.get_webdav_client()
    
    remote_collection_path = f'{constants.MDR_PROJECT_ID}/{constants.MDR_COLLECTION_ID}'
    object_names = client.list(remote_collection_path)
    remote_zip_file_paths = []
    for object_name in object_names:
        if object_name.startswith('PORSCH'):
            remote_hospital_path = f'{remote_collection_path}/{object_name}'
            zip_file_names = client.list(remote_hospital_path)
            for zip_file_name in zip_file_names:
                zip_file_path = f'{remote_hospital_path}{zip_file_name}'
                remote_zip_file_paths.append(zip_file_path)
            print(object_name)

    folder_mapping = utils.load_dictionary('foldermapping.json', reverse=True)
    
    file_mapping = {}
    for remote_zip_file_path in remote_zip_file_paths:
        patient_id = os.path.split(remote_zip_file_path)[1][:-4]
        if utils.is_number(patient_id):            
            remote_hospital_name = utils.get_parent_name(remote_zip_file_path)
            local_hospital_name = folder_mapping[remote_hospital_name]
            local_l3_file_name = f'{patient_id}.dcm'
            local_l3_file_path = os.path.join(constants.LOCAL_L3_DIR, local_hospital_name, local_l3_file_name)
            if os.path.exists(local_l3_file_path):
                file_mapping[local_l3_file_path] = remote_zip_file_path

    utils.save_dictionary(file_mapping, 'filemapping.json')


if __name__ == '__main__':
    main()


import os
import pandas as pd


def main():
    df = pd.read_excel('C:\\Users\\r.brecheisen\\OneDrive - Maastricht University\\Research\\Documents\\Projects\\IMAGIO\\Vaughtey CT levers\\Selectie scans.xlsx', sheet_name='400 selectie')
    df_all_phases = df[(df['Procedure RIS'] == 'CT Abdomen') & (df['art'] == 1) & (df['blanco'] == 1) & (df['Pvp'] == 1)]
    print(f'Found {df_all_phases.shape[0]} rows')
    with open('accessionnumbers.txt', 'w') as f:
        for idx, row in df_all_phases.iterrows():
            f.write(row['Onderzoeksnummer'] + '\n')

if __name__ == '__main__':
    main()


import os
import shutil
import pydicom
import numpy as np
import nibabel as nib
from nibabel.processing import resample_from_to
from totalsegmentator.python_api import totalsegmentator

DICOM_DIR = 'D:\\Mosamatic\\Dixon\\AllFiles'
DICOM_DIXON_IN_PHASE_DIR = 'D:\\Mosamatic\\Dixon\\InPhase'
DICOM_DIXON_OPPOSITE_PHASE_DIR = 'D:\\Mosamatic\\Dixon\\OppositePhase'
TOTALSEG_OUTPUT_DIR = 'D:\\Mosamatic\\Dixon\\TotalSegmentator'
TOTALSEG_TASK = 'total_mr'
TOTALSEG_LIVER_MASK_FILE = 'liver.nii.gz'
PDFF_DIR = 'D:\\Mosamatic\\Dixon\\Pdff'
PDFF_MAP_FILE = 'pdff.nii.gz'
PDFF_LIVER_MAP_FILE = 'liver_pdff.nii.gz'


def extract_and_copy_dixon_images():
    os.makedirs(DICOM_DIXON_IN_PHASE_DIR, exist_ok=True)
    os.makedirs(DICOM_DIXON_OPPOSITE_PHASE_DIR, exist_ok=True)
    for f in os.listdir(DICOM_DIR):
        f_path = os.path.join(DICOM_DIR, f)
        p = pydicom.dcmread(f_path, stop_before_pixels=True)
        if p.Modality == 'MR':
            if 'ImageType' in p and len(p.ImageType) > 4:
                if p.ImageType[4] == 'IN_PHASE':
                    shutil.copy(f_path, DICOM_DIXON_IN_PHASE_DIR)
                elif p.ImageType[4] == 'OPP_PHASE':
                    shutil.copy(f_path, DICOM_DIXON_OPPOSITE_PHASE_DIR)
                else:
                    pass
    nr_in_phase = len(os.listdir(DICOM_DIXON_IN_PHASE_DIR))
    nr_opposite_phase = len(os.listdir(DICOM_DIXON_OPPOSITE_PHASE_DIR))
    print(f'Extracted Dixon in-phase ({nr_in_phase}) and opposite-phase ({nr_opposite_phase}) images')


def build_affine(p_list):
    first = p_list[0]
    px_spacing = [float(x) for x in first.PixelSpacing]
    slice_thickness = float(getattr(first, "SliceThickness", 1.0))
    orient = [float(x) for x in first.ImageOrientationPatient]
    row_cosine = np.array(orient[0:3])
    col_cosine = np.array(orient[3:6])
    slice_cosine = np.cross(row_cosine, col_cosine)
    origin = np.array([float(x) for x in first.ImagePositionPatient])
    direction = np.vstack((row_cosine, col_cosine, slice_cosine)).T
    spacing = np.array([px_spacing[0], px_spacing[1], slice_thickness])
    affine = np.eye(4)
    affine[:3, :3] = direction * spacing
    affine[:3, 3] = origin
    return affine


def load_numpy_volume_from_dicom_dir(dicom_dir):
    files = []
    for f in os.listdir(dicom_dir):
        files.append(os.path.join(dicom_dir, f))
    p_list = [pydicom.dcmread(f) for f in files]
    sorted_p_list = sorted(p_list, key=lambda p: p.InstanceNumber)
    affine = build_affine(sorted_p_list)
    volume = []
    for p in sorted_p_list:
        arr = p.pixel_array.astype(np.float32)
        if 'RescaleSlope' in p and 'RescaleIntercept' in p:
            arr = arr * float(p.RescaleSlope) + float(p.RescaleIntercept)
        volume.append(arr)    
    return np.stack(volume, axis=0), affine


def load_volume_from_dcm2niix(input_dir, output_dir, file_name):
    os.system(f'dcm2niix -z y -f {file_name} -o {output_dir} {input_dir}')
    save_file = os.path.join(output_dir, f'{file_name}_phMag.nii.gz')
    file_path = os.path.join(output_dir, f'{file_name}.nii.gz')
    os.rename(save_file, file_path)
    return file_path


def run_totalsegmentator_on_dixon_opposite_phase_images():
    if os.path.isdir(TOTALSEG_OUTPUT_DIR):
        shutil.rmtree(TOTALSEG_OUTPUT_DIR)
    # Total segmentator works best when loading DICOM images from a directory
    totalsegmentator(input=DICOM_DIXON_OPPOSITE_PHASE_DIR, output=TOTALSEG_OUTPUT_DIR, task=TOTALSEG_TASK)


def calculate_pdff_map():
    if os.path.isdir(PDFF_DIR):
        shutil.rmtree(PDFF_DIR)
    os.makedirs(PDFF_DIR, exist_ok=True)
    ip_file = load_volume_from_dcm2niix(DICOM_DIXON_OPPOSITE_PHASE_DIR, PDFF_DIR, 'ip')
    op_file = load_volume_from_dcm2niix(DICOM_DIXON_OPPOSITE_PHASE_DIR, PDFF_DIR, 'op')
    ip_volume = nib.load(ip_file)
    op_volume = nib.load(op_file)
    affine = ip_volume.affine
    ip_volume = ip_volume.get_fdata()
    op_volume = op_volume.get_fdata()
    water_volume = 0.5 * (ip_volume + op_volume)
    fat_volume = 0.5 * np.abs(ip_volume - op_volume)
    pdff_map = (fat_volume / (water_volume + fat_volume + 1e-6)) * 100
    return pdff_map, affine


def apply_liver_mask_to_pdff_map(pdff_map, affine):
    pdff_img = nib.Nifti1Image(pdff_map.astype(np.float32), affine)
    pdff_img_file = os.path.join(PDFF_DIR, 'pdff.nii.gz')
    nib.save(pdff_img, pdff_img_file)
    liver_mask_img = nib.load(os.path.join(TOTALSEG_OUTPUT_DIR, 'liver.nii.gz'))
    liver_mask_resampled_img = resample_from_to(liver_mask_img, pdff_img, order=0)
    liver_mask_volume = liver_mask_resampled_img.get_fdata()
    liver_pdff_volume = np.where(liver_mask_volume > 0, pdff_map, np.nan)
    liver_pdff_img = nib.Nifti1Image(liver_pdff_volume.astype(np.float32), affine)
    liver_pdff_img_file = os.path.join(PDFF_DIR, 'liver_pdff.nii.gz')
    nib.save(liver_pdff_img, liver_pdff_img_file)


def test_totalseg_dixon():
    # extract_and_copy_dixon_images()
    # run_totalsegmentator_on_dixon_opposite_phase_images()
    pdff_map, affine = calculate_pdff_map()
    apply_liver_mask_to_pdff_map(pdff_map, affine)


import os
import vtk
import platform
import pandas as pd

PLATFORM = platform.system()
if PLATFORM == 'Windows':
    BASE_DIR_PATH = 'G:/My Drive'
elif PLATFORM == 'Darwin':
    BASE_DIR_PATH = '/Users/ralph/Library/CloudStorage/GoogleDrive-ralph.brecheisen@gmail.com/My Drive'
SEGMENTS_DIR_PATH = f'{BASE_DIR_PATH}/data/mosamatic2/data/liveranalysispipeline/totalsegmentatortask'
SEGMENTS_STATISTICS_FILE_PATH = f'{BASE_DIR_PATH}/data/mosamatic2/data/liveranalysispipeline/calculatemaskstatisticstask/statistics.csv'

SEGMENT_COLORS = [
    (0.80, 0.25, 0.25),  # muted red
    (0.25, 0.55, 0.80),  # steel blue
    (0.30, 0.70, 0.45),  # green
    (0.85, 0.65, 0.25),  # amber
    (0.55, 0.40, 0.75),  # purple
    (0.90, 0.45, 0.15),  # orange
    (0.20, 0.75, 0.75),  # teal
    (0.75, 0.75, 0.30),  # olive
]

OPACITY_SELECTED = 1.0
OPACITY_NOT_SELECTED = 0.25


class MultiActorPicker:
    def __init__(self, renderer, interactor, actors, volumes, selected_volume_actor):
        self.ren = renderer
        self.iren = interactor
        self.actors = actors
        self.actor_keys = {v: k for k, v in actors.items()}
        self.volumes = volumes
        self.selected_volume_actor = selected_volume_actor
        self.selected = set()
        self._orig = {}

        self.picker = vtk.vtkCellPicker()
        self.picker.SetTolerance(0.001)

        # Use trackball so it rotates only when you move
        self.style = vtk.vtkInteractorStyleTrackballCamera()
        self.iren.SetInteractorStyle(self.style)

        # Hook events
        self.iren.AddObserver("LeftButtonPressEvent", self.on_left_down)
        self.iren.AddObserver("LeftButtonReleaseEvent", self.on_left_up)

        self._allow_camera_drag = True

    def _save(self, a):
        if a in self._orig:
            return

    def _highlight(self, a, on):
        self._save(a)
        p = a.GetProperty()
        if on:
            p.SetOpacity(OPACITY_SELECTED)
        else:
            p.SetOpacity(OPACITY_NOT_SELECTED)

    def clear(self):
        for a in list(self.selected):
            self._highlight(a, False)
        self.selected.clear()

    def toggle(self, a):
        if a in self.selected:
            self._highlight(a, False)
            self.selected.remove(a)
        else:
            self._highlight(a, True)
            self.selected.add(a)
        total_selected_volume = 0
        for a in self.selected:
            a_key = self.actor_keys[a]
            volume = int(self.volumes[a_key])
            total_selected_volume += volume
        self.selected_volume_actor.SetInput(f'Selected volume: {total_selected_volume} mL')
            

    def on_left_down(self, iren, evt):
        x, y = iren.GetEventPosition()

        # Only do picking when Shift or Ctrl is held.
        do_pick = (iren.GetShiftKey() or iren.GetControlKey())

        if do_pick:
            self.picker.Pick(x, y, 0, self.ren)
            a = self.picker.GetActor()
            a_key = self.actor_keys[a]

            if a in self.actors.values():
                # Additive (Shift/Ctrl): toggle selection
                self.toggle(a)
            else:
                # Shift/Ctrl + empty click: clear selection (optional)
                self.clear()

            iren.GetRenderWindow().Render()
            self._allow_camera_drag = False  # don't also start a camera drag
            return

        # No modifier => normal rotate/pan/zoom interaction
        self._allow_camera_drag = True
        self.style.OnLeftButtonDown()

    def on_left_up(self, iren, evt):
        if self._allow_camera_drag:
            self.style.OnLeftButtonUp()


def labelmap_to_surface(image: vtk.vtkImageData, label=1) -> vtk.vtkPolyData:
    dmc = vtk.vtkDiscreteMarchingCubes()
    dmc.SetInputData(image)
    dmc.SetValue(0, label)
    dmc.Update()
    cleaner = vtk.vtkCleanPolyData()
    cleaner.SetInputConnection(dmc.GetOutputPort())
    cleaner.Update()
    smoother = vtk.vtkWindowedSincPolyDataFilter()
    smoother.SetInputConnection(cleaner.GetOutputPort())
    smoother.SetNumberOfIterations(20)   # 10–30 typical
    smoother.SetPassBand(0.1)              # smaller = smoother (e.g. 0.05–0.2)
    smoother.SetFeatureAngle(120.0)      # preserve sharper edges
    smoother.BoundarySmoothingOff()              # usually best for closed organs
    smoother.FeatureEdgeSmoothingOff()           # keep features; turn On if you want more smoothing
    smoother.NonManifoldSmoothingOn()
    smoother.NormalizeCoordinatesOn()            # helps numerical stability
    smoother.Update()
    normals = vtk.vtkPolyDataNormals()
    normals.SetInputConnection(smoother.GetOutputPort())
    normals.SetComputePointNormals(True)
    normals.SetComputeCellNormals(False)
    normals.SetAutoOrientNormals(True)
    normals.SetConsistency(True)
    normals.Update()
    return normals.GetOutput()


def polydata_actor(polydata: vtk.vtkPolyData, opacity=1.0) -> vtk.vtkActor:
    mapper = vtk.vtkPolyDataMapper()
    mapper.SetInputData(polydata)
    mapper.ScalarVisibilityOff()
    actor = vtk.vtkActor()
    actor.SetMapper(mapper)
    return actor


def build_liver_volume_dict(f_path):
    volumes = {}
    df = pd.read_csv(f_path, sep=';')
    for _, row in df.iterrows():
        volumes[row['file']] = row['volume_mL']
    return volumes


def main():
    liver_segments_dir_path = SEGMENTS_DIR_PATH
    liver_segments_stats_file_path = SEGMENTS_STATISTICS_FILE_PATH
    volumes = build_liver_volume_dict(liver_segments_stats_file_path)
    total_volume = sum(volumes.values())

    renderer = vtk.vtkRenderer()
    render_window = vtk.vtkRenderWindow()
    render_window.AddRenderer(renderer)
    interactor = vtk.vtkRenderWindowInteractor()
    interactor.SetRenderWindow(render_window)
    interactor.SetInteractorStyle(vtk.vtkInteractorStyleTrackballCamera())

    selected_volume_actor = vtk.vtkTextActor()
    selected_volume_actor.SetInput('Selected volume: 0 mL')
    selected_volume_actor.GetTextProperty().SetFontSize(18)
    selected_volume_actor.GetTextProperty().SetColor(1, 1, 1)  # RGB 0–1
    selected_volume_actor.SetDisplayPosition(20, 50)

    total_volume_actor = vtk.vtkTextActor()
    total_volume_actor.SetInput(f'Total volume: {total_volume} mL')
    total_volume_actor.GetTextProperty().SetFontSize(18)
    total_volume_actor.GetTextProperty().SetColor(1, 1, 1)  # RGB 0–1
    total_volume_actor.SetDisplayPosition(20, 20)

    renderer.AddViewProp(total_volume_actor)
    renderer.AddViewProp(selected_volume_actor)

    i = 0
    actors = {}
    for f in os.listdir(liver_segments_dir_path):
        segment_name = f
        f_path = os.path.join(liver_segments_dir_path, f)
        reader = vtk.vtkNIFTIImageReader()
        reader.SetFileName(f_path)
        reader.Update()
        image = reader.GetOutput()
        surface = labelmap_to_surface(image)
        actor = polydata_actor(surface, 1.0)
        prop = actor.GetProperty()
        prop.SetSpecular(0.3)
        prop.SetSpecularPower(20)
        prop.SetDiffuse(0.7)
        prop.SetAmbient(0.1)
        prop.SetColor(*SEGMENT_COLORS[i])
        prop.SetOpacity(0.5)
        renderer.AddActor(actor)
        actors[segment_name] = actor
        i += 1

    picker = MultiActorPicker(renderer, interactor, actors, volumes, selected_volume_actor)

    renderer.SetBackground(0.1, 0.1, 0.12)
    renderer.ResetCamera()
    render_window.SetSize(1200, 900)
    render_window.Render()
    interactor.Start()


if __name__ == '__main__':
    main()


import pydicom
import numpy as np
from pathlib import Path
from typing import Tuple
from pydicom.multival import MultiValue
from PySide6.QtGui import (
    QImage,
)


#-------------------------------------------------------------------------------------------------------
def clamp(v: int, lo: int, hi: int) -> int:
    return lo if v < lo else hi if v > hi else v


#-------------------------------------------------------------------------------------------------------
def circle_brush(radius: int) -> np.ndarray:
    """Boolean (2r+1, 2r+1) circular kernel."""
    r = max(1, int(radius))
    yy, xx = np.ogrid[-r : r + 1, -r : r + 1]
    return (xx * xx + yy * yy) <= (r * r)


#-------------------------------------------------------------------------------------------------------
def np_u8_gray_to_qimage(gray: np.ndarray) -> QImage:
    """gray: (H, W) uint8 -> QImage Format_Grayscale8 (shares memory!)."""
    if gray.dtype != np.uint8 or gray.ndim != 2:
        raise ValueError("Expected uint8 (H,W)")
    h, w = gray.shape
    # Important: keep a reference to `gray` alive as long as QImage is used.
    return QImage(gray.data, w, h, int(gray.strides[0]), QImage.Format_Grayscale8)


#-------------------------------------------------------------------------------------------------------
def np_rgba_to_qimage(rgba: np.ndarray) -> QImage:
    """rgba: (H, W, 4) uint8 -> QImage Format_RGBA8888 (shares memory!)."""
    if rgba.dtype != np.uint8 or rgba.ndim != 3 or rgba.shape[2] != 4:
        raise ValueError("Expected uint8 (H,W,4)")
    h, w, _ = rgba.shape
    return QImage(rgba.data, w, h, int(rgba.strides[0]), QImage.Format_RGBA8888)


#-------------------------------------------------------------------------------------------------------
def dicom_to_image_and_u8_display(path: Path) -> Tuple[np.ndarray, np.ndarray, dict]:
    """
    Read DICOM single-frame image -> display uint8 (H,W).
    Returns (disp8, meta). This is just for visualization; you keep original as needed.
    """
    if pydicom is None:
        raise RuntimeError("pydicom not installed. `pip install pydicom`")

    ds = pydicom.dcmread(str(path))
    img = ds.pixel_array.astype(np.float32)

    slope = float(getattr(ds, "RescaleSlope", 1.0))
    intercept = float(getattr(ds, "RescaleIntercept", 0.0))
    img = img * slope + intercept

    # Window/level if present
    wc = getattr(ds, "WindowCenter", None)
    ww = getattr(ds, "WindowWidth", None)

    def _mv_first(x):
        if isinstance(x, MultiValue):
            return float(x[0])
        return float(x)

    if wc is not None and ww is not None:
        c = _mv_first(wc)
        w = _mv_first(ww)
        lo = c - (w / 2.0)
        hi = c + (w / 2.0)
    else:
        lo, hi = np.percentile(img, (1, 99))
        if hi <= lo:
            lo, hi = float(img.min()), float(img.max() + 1e-6)

    disp = np.clip((img - lo) / (hi - lo), 0.0, 1.0)
    disp8 = (disp * 255.0).astype(np.uint8)

    meta = {
        "SOPInstanceUID": getattr(ds, "SOPInstanceUID", ""),
        "SeriesInstanceUID": getattr(ds, "SeriesInstanceUID", ""),
        "StudyInstanceUID": getattr(ds, "StudyInstanceUID", ""),
        "PixelSpacing": [float(x) for x in getattr(ds, "PixelSpacing", [])] if hasattr(ds, "PixelSpacing") else [],
        "Shape": list(disp8.shape),
    }
    return img, disp8, meta


import click

from mosamatic2.core.tasks import SegmentMuscleFatL3TensorFlowTask


@click.command(help='Extracts muscle and fat regions from CT images at L3 (uses TensorFlow)')
@click.option(
    '--images', 
    required=True, 
    type=click.Path(), 
    help='Input directory with images'
)
@click.option(
    '--model_files', 
    required=True, 
    type=click.Path(), 
    help='Input directory with AI model files'
)
@click.option(
    '--output', 
    required=True, 
    type=click.Path(), 
    help='Output directory'
)
@click.option(
    '--overwrite', 
    default=False, 
    type=click.BOOL, 
    help='Overwrite [true|false]'
)
def segmentmusclefatl3tensorflow(images, model_files, output, overwrite):
    """
    Automatically segments muscle and fat tissue in CT images at L3 level.
    
    Parameters
    ----------
    --images : str
        Directory with input L3 images

    --model_files : str
        Directory with AI model files (model-1.0.zip, contour_model-1.0.zip, params-1.0.json)

    --output : str
        Path to output directory

    --overwrite : bool
        Overwrite contents output directory [true|false]
    """
    task = SegmentMuscleFatL3TensorFlowTask(
        inputs={'images': images, 'model_files': model_files},
        params={'model_version': 1.0},
        output=output,
        overwrite=overwrite,
    )
    task.run()


# %%
import os
import pathlib
import h5py

import matplotlib.pyplot as plt
import numpy as np
import torch

# from models_ import AttentionUNet, UNet
from param_loader import Params

from models import UNet
from models2 import AttentionUNet


def normalize(data: np.ndarray, lower_bound: int, upper_bound: int) -> np.ndarray:
    """Normalize input: 'data' between 'lower_bound' and 'upper_bound',
    and scale between [0, 1].

    Parameters
    ----------
    data : np.ndarray
        Input image
    lower_bound : int
        Minimum Hounsfield Unit
    upper_bound : int
        Maximum Hounsfield Unit

    Returns
    -------
    np.ndarray
        Normalized and scaled input
    """

    data = (data - lower_bound) / (upper_bound - lower_bound)
    data[data > 1] = 1
    data[data < 0] = 0
    return data


DEVICE = "cuda"

# Load params.json
param_path = pathlib.Path(os.getcwd()) / "params.json"

params = Params(param_path)

# Load Contour model
model_path = os.path.join(
    os.getcwd(),
    "assets/logs/20250325-122319Contour/saved_models/highest_val.pt",
)

model_c = UNet(params, 2).to(device=DEVICE)
model_c.load_state_dict(torch.load(model_path, weights_only=False))
model_c.eval()

# Data path
test_path = os.path.join(
    os.getcwd(),
    "assets/data/Testing.h5",
)

# Load data
with h5py.File(test_path, "r") as f:
    a_group_key2 = list(f.keys())
    pad_width = 0
    images = np.zeros(
        [len(a_group_key2), 512 + pad_width, 512 + pad_width], dtype=np.float32
    )
    labels = np.zeros(
        [len(a_group_key2), 512 + pad_width, 512 + pad_width], dtype=np.int16
    )

    # List to store keys of skipped patients
    skipped_keys = []
    for i, name in enumerate(a_group_key2):
        if i % 500 == 0:
            print(f"Loading patients: {i} out of {len(a_group_key2)}")
        image = f[str(name)]["images"][()]
        label = f[str(name)]["labels"][()]

        image = normalize(
            image,
            lower_bound=params.dict["lower_bound"],
            upper_bound=params.dict["upper_bound"],
        )
        label[label == 5] = 2
        label[label == 7] = 3
        label[label > 3] = 0
        images[i, :, :] = image
        labels[i, :, :] = np.uint8(label)

    print(">>> Unique labels in {}: {}".format(test_path, np.unique(labels)))

# Take example patient
layer = 100
image = images[layer, :, :]
label = labels[layer, :, :]

# Predict contour
with torch.no_grad():
    # Create 4D Tensor input
    input = np.expand_dims(image, 0)
    input = np.expand_dims(input, 0)
    input = torch.Tensor(input)
    input = input.to(DEVICE, dtype=torch.float)

    # Predict
    prediction = model_c(input)
    prediction = torch.argmax(prediction, axis=1)
    prediction = prediction.squeeze()
    prediction = prediction.detach().cpu().numpy()

# Keep only torso
image *= prediction

# Remove contour model from memory if necessary
# del model_c

# Load body composition model
model_path = os.path.join(
    os.getcwd(),
    "assets/logs/20250516-085253-L3/saved_models/highest_val.pt",
)
model = UNet(params, 4).to(device=DEVICE)
model.load_state_dict(torch.load(model_path, weights_only=False))
model.eval()

# Predict body composition
with torch.no_grad():
    input = np.expand_dims(image, 0)
    input = np.expand_dims(input, 0)
    input = torch.Tensor(input)

    input = input.to(DEVICE, dtype=torch.float)

    prediction = model(input)
    prediction = torch.argmax(prediction, axis=1)
    prediction = prediction.squeeze()
    prediction = prediction.detach().cpu().numpy()

# Duplicate, taken from Leuven, not used.
prediction_no_vat = np.copy(prediction)
# prediction_no_vat[prediction_no_vat == 2] = 0

# Plt example
fig, ax = plt.subplots(1, 4, figsize=(20, 20))
ax[0].imshow(image, cmap='gray')
ax[1].imshow(label)
ax[1].title.set_text("Label")
ax[2].imshow(prediction)
ax[2].title.set_text("Prediction")
ax[3].imshow(prediction_no_vat)
ax[3].title.set_text("Duplicated, to be removed")

# %%



from PySide6.QtCore import Qt, Signal, QSignalBlocker
from PySide6.QtWidgets import (
    QWidget,
    QGroupBox,
    QVBoxLayout,
    QHBoxLayout,
    QGridLayout,
    QSpinBox,
    QLabel,
    QPushButton,
    QSlider,
    QCheckBox,
    QGridLayout,
    QRadioButton,
    QButtonGroup,
    QFrame,
)

HELP_INFO = """
<b>Shortcuts</b>
<table>
    <tr><td>Brush radius:</td><td>CTRL + mouse wheel</td></tr>
    <tr><td>Active labels:</td><td>
        "1" for muscle<br>
        "5" for visceral fat<br>
        "7" for subcutaneous fat
    </td></tr>
    <tr><td>Zooming</td><td>Mouse wheel</td></tr>
    <tr><td>Zoom rectangle</td><td>Hold "Z" + mouse click and drag</td></tr>
    <tr><td>Panning</td><td>SPACE + mouse click and drag</td></tr>
    <tr><td>Undo</td><td>
        CTRL + "Z"<br>
        "U"
    </td></tr>
    <tr><td>Remove annotation</td><td>
        "E" + mouse click and drag<br>
        Right mouse button click and drag
    </td></tr>
    <tr><td>Toggle SmartPaint</td><td>"S"</td></tr>
    <tr><td>Toggle fix non-active labels</td><td>"F"</td></tr>
</table>
"""


class SegmentationEditorControls(QWidget):

    # SIGNALS

    image_loaded = Signal()
    segmentation_loaded = Signal()
    segmentation_saved = Signal()
    active_label_changed = Signal(int)
    smart_paint_changed = Signal(bool)
    fix_non_active_labels_changed = Signal(bool)
    hu_lo_muscle_changed = Signal(int)
    hu_hi_muscle_changed = Signal(int)
    hu_lo_vat_changed = Signal(int)
    hu_hi_vat_changed = Signal(int)
    hu_lo_sat_changed = Signal(int)
    hu_hi_sat_changed = Signal(int)
    overall_opacity_changed = Signal(float)
    clear_labels = Signal()
    reset_zoom = Signal()

    #-------------------------------------------------------------------------------------------------------
    def __init__(self, 
                 active_label, 
                 brush_radius, 
                 smart_paint, 
                 fix_non_active_labels,
                 overall_opacity,
                 hu_lo_muscle,
                 hu_hi_muscle,
                 hu_lo_vat,
                 hu_hi_vat,
                 hu_lo_sat,
                 hu_hi_sat,
                 parent,
        ):
        super(SegmentationEditorControls, self).__init__(parent)

        # Fields
        self._active_label = active_label
        self._brush_radius = brush_radius
        self._smart_paint = smart_paint
        self._fix_non_active_labels = fix_non_active_labels
        self._overall_opacity = overall_opacity
        self._hu_lo_muscle = hu_lo_muscle
        self._hu_hi_muscle = hu_hi_muscle
        self._hu_lo_vat = hu_lo_vat
        self._hu_hi_vat = hu_hi_vat
        self._hu_lo_sat = hu_lo_sat
        self._hu_hi_sat = hu_hi_sat

        # UI widgets
        self._load_image_button = None
        self._load_segmentation_button = None
        self._save_segmentation_button = None
        self._active_label_group = None
        self._fix_non_active_labels_checkbox = None
        self._hu_hi_range_muscle_spinbox = None
        self._hu_lo_range_muscle_spinbox = None
        self._hu_hi_range_vat_spinbox = None
        self._hu_lo_range_vat_spinbox = None
        self._hu_hi_range_sat_spinbox = None
        self._hu_lo_range_sat_spinbox = None
        self._brush_radius_label = None
        self._overall_opacity_label = None
        self._overall_opacity_slider = None
        self._reset_zoom_button = None
        self._clear_all_labels_button = None
        self._help_info_label = None
        self.init()

    # INITIALIZATION

    #-------------------------------------------------------------------------------------------------------
    def init(self):

        # Loading image and saving segmentation
        self._load_image_button = QPushButton('Load DICOM image...')
        self._load_image_button.clicked.connect(self.handle_load_image_button)
        self._load_segmentation_button = QPushButton('Load segmentation...')
        self._load_segmentation_button.clicked.connect(self.handle_load_segmentation_button)
        self._save_segmentation_button = QPushButton('Save segmentation...')
        self._save_segmentation_button.clicked.connect(self.handle_save_segmentation_button)

        # Active label groupbox and radiobuttons
        active_label_groupbox = QGroupBox('Active label')
        muscle_radio_button = QRadioButton('Muscle (1)')
        vat_radio_button = QRadioButton('Visceral fat (5)')
        sat_radio_button = QRadioButton('Subcutaneous fat (7)')
        for rb in [muscle_radio_button, vat_radio_button, sat_radio_button]:
            if str(self._active_label) in rb.text():
                rb.setChecked(True)
            else:
                rb.setChecked(False)

        # Active label layout
        active_label_layout = QVBoxLayout(active_label_groupbox)
        active_label_layout.addWidget(muscle_radio_button)
        active_label_layout.addWidget(vat_radio_button)
        active_label_layout.addWidget(sat_radio_button)

        # Active label group
        self._active_label_group = QButtonGroup(active_label_groupbox)
        self._active_label_group.setExclusive(True)
        self._active_label_group.addButton(muscle_radio_button, 1)
        self._active_label_group.addButton(vat_radio_button, 5)
        self._active_label_group.addButton(sat_radio_button, 7)
        self._active_label_group.idClicked.connect(self.handle_active_label_changed)

        # Fix non-active labels
        self._fix_non_active_labels_checkbox = QCheckBox('Fix non-active labels', checked=self._fix_non_active_labels)
        self._fix_non_active_labels_checkbox.checkStateChanged.connect(self.handle_fix_non_active_labels_changed)

        # HU range groupbox and widgets
        hu_range_groupbox = QGroupBox('Hounsfield Unit ranges')
        self._hu_lo_range_muscle_spinbox = QSpinBox(self, minimum=-200, maximum=200, value=self._hu_lo_muscle)
        self._hu_lo_range_muscle_spinbox.valueChanged.connect(self.handle_hu_lo_range_muscle_changed)
        self._hu_hi_range_muscle_spinbox = QSpinBox(self, minimum=-200, maximum=200, value=self._hu_hi_muscle)
        self._hu_hi_range_muscle_spinbox.valueChanged.connect(self.handle_hu_hi_range_muscle_changed)
        self._hu_lo_range_vat_spinbox = QSpinBox(self, minimum=-200, maximum=200, value=self._hu_lo_vat)
        self._hu_lo_range_vat_spinbox.valueChanged.connect(self.handle_hu_lo_range_vat_changed)
        self._hu_hi_range_vat_spinbox = QSpinBox(self, minimum=-200, maximum=200, value=self._hu_hi_vat)
        self._hu_hi_range_vat_spinbox.valueChanged.connect(self.handle_hu_hi_range_vat_changed)
        self._hu_lo_range_sat_spinbox = QSpinBox(self, minimum=-200, maximum=200, value=self._hu_lo_sat)
        self._hu_lo_range_sat_spinbox.valueChanged.connect(self.handle_hu_lo_range_sat_changed)
        self._hu_hi_range_sat_spinbox = QSpinBox(self, minimum=-200, maximum=200, value=self._hu_hi_sat)
        self._hu_hi_range_sat_spinbox.valueChanged.connect(self.handle_hu_hi_range_sat_changed)

        # HU range layout
        hu_range_layout = QGridLayout(hu_range_groupbox)
        hu_range_layout.addWidget(QLabel(''), 0, 0)
        hu_range_layout.addWidget(QLabel('Low'), 0, 1)
        hu_range_layout.addWidget(QLabel('High'), 0, 2)
        hu_range_layout.addWidget(QLabel('Muscle'), 1, 0)
        hu_range_layout.addWidget(self._hu_lo_range_muscle_spinbox, 1, 1)
        hu_range_layout.addWidget(self._hu_hi_range_muscle_spinbox, 1, 2)
        hu_range_layout.addWidget(QLabel('Visceral fat'), 2, 0)
        hu_range_layout.addWidget(self._hu_lo_range_vat_spinbox, 2, 1)
        hu_range_layout.addWidget(self._hu_hi_range_vat_spinbox, 2, 2)
        hu_range_layout.addWidget(QLabel('Subcutaneous fat'), 3, 0)
        hu_range_layout.addWidget(self._hu_lo_range_sat_spinbox, 3, 1)
        hu_range_layout.addWidget(self._hu_hi_range_sat_spinbox, 3, 2)

        # Separator line
        line = QFrame()
        line.setFrameShape(QFrame.Shape.HLine)
        line.setFrameShadow(QFrame.Shadow.Plain)

        # Smart paint
        self._smart_paint_checkbox = QCheckBox('Smart paint', checked=self._smart_paint)
        self._smart_paint_checkbox.checkStateChanged.connect(self.handle_smart_paint_changed)

        # Brush radius
        self._brush_radius_label = QLabel(f'Brush radius: {str(self._brush_radius)}')
        
        # Overall opacity
        self._overall_opacity_slider = QSlider(Qt.Horizontal)
        self._overall_opacity_slider.setRange(0, 100)
        self._overall_opacity_slider.setValue(int(self._overall_opacity * 100))
        self._overall_opacity_slider.valueChanged.connect(self.handle_overall_opacity_changed)
        self._overall_opacity_label = QLabel(str(self._overall_opacity))
        overall_opacity_layout = QHBoxLayout()
        overall_opacity_layout.addWidget(self._overall_opacity_slider)
        overall_opacity_layout.addWidget(self._overall_opacity_label)
        
        # Reset zoom
        self._reset_zoom_button = QPushButton('Reset zoom')
        self._reset_zoom_button.clicked.connect(self.handle_reset_zoom_button)
        
        # Clear all labels
        self._clear_all_labels_button = QPushButton('Clear all labels')
        self._clear_all_labels_button.clicked.connect(self.handle_clear_all_labels_button)

        # Help info
        self._help_info_label = QLabel(HELP_INFO)
        self._help_info_label.setWordWrap(True)
        self._help_info_label.setTextFormat(Qt.RichText)
        self._help_info_label.setAlignment(Qt.AlignTop | Qt.AlignLeft)

        # Main layout
        layout = QVBoxLayout()
        layout.setAlignment(Qt.AlignmentFlag.AlignTop)
        layout.addWidget(self._load_image_button)
        layout.addWidget(self._save_segmentation_button)
        layout.addWidget(active_label_groupbox)
        layout.addWidget(hu_range_groupbox)
        layout.addWidget(self._fix_non_active_labels_checkbox)
        layout.addWidget(self._smart_paint_checkbox)
        layout.addWidget(line)
        layout.addWidget(QLabel('Overall opacity'))
        layout.addLayout(overall_opacity_layout)
        layout.addWidget(self._brush_radius_label)
        layout.addWidget(self._reset_zoom_button)
        layout.addWidget(self._clear_all_labels_button)
        layout.addWidget(self._help_info_label)
        self.setLayout(layout)

    # GETTERS

    #-------------------------------------------------------------------------------------------------------
    def active_label(self):
        return self._active_label
    
    #-------------------------------------------------------------------------------------------------------
    def set_active_label(self, active_label):
        self._active_label = active_label
        button = self._active_label_group.button(self._active_label)
        with QSignalBlocker(button):
            button.setChecked(True)
    
    #-------------------------------------------------------------------------------------------------------
    def brush_radius(self):
        return self._brush_radius
    
    #-------------------------------------------------------------------------------------------------------
    def set_brush_radius(self, brush_radius):
        self._brush_radius = brush_radius
        self._brush_radius_label.setText(f'Brush radius: {str(self._brush_radius)}')
    
    #-------------------------------------------------------------------------------------------------------
    def smart_paint(self):
        return self._smart_paint

    #-------------------------------------------------------------------------------------------------------
    def set_smart_paint(self, smart_paint):
        self._smart_paint = smart_paint
        with QSignalBlocker(self._smart_paint_checkbox):
            self._smart_paint_checkbox.setChecked(self._smart_paint)
    
    #-------------------------------------------------------------------------------------------------------
    def fix_non_active_labels(self):
        return self._fix_non_active_labels
    
    #-------------------------------------------------------------------------------------------------------
    def set_fix_non_active_labels(self, fix_non_active_labels):
        self._fix_non_active_labels = fix_non_active_labels
        with QSignalBlocker(self._fix_non_active_labels_checkbox):
            self._fix_non_active_labels_checkbox.setChecked(self._fix_non_active_labels)
    
    #-------------------------------------------------------------------------------------------------------
    def overall_opacity(self):
        return self._overall_opacity
    
    #-------------------------------------------------------------------------------------------------------
    def hu_lo_muscle(self):
        return self._hu_lo_muscle
    
    #-------------------------------------------------------------------------------------------------------
    def hu_hi_muscle(self):
        return self._hu_hi_muscle
    
    #-------------------------------------------------------------------------------------------------------
    def hu_lo_vat(self):
        return self._hu_lo_vat
    
    #-------------------------------------------------------------------------------------------------------
    def hu_hi_vat(self):
        return self._hu_hi_vat
    
    #-------------------------------------------------------------------------------------------------------
    def hu_lo_sat(self):
        return self._hu_lo_sat
    
    #-------------------------------------------------------------------------------------------------------
    def hu_hi_sat(self):
        return self._hu_hi_sat
    
    # EVENT HANDLERS

    #-------------------------------------------------------------------------------------------------------
    def handle_load_image_button(self):
        self.image_loaded.emit()

    #-------------------------------------------------------------------------------------------------------
    def handle_load_segmentation_button(self):
        self.segmentation_loaded.emit()

    #-------------------------------------------------------------------------------------------------------
    def handle_save_segmentation_button(self):
        self.segmentation_saved.emit()

    #-------------------------------------------------------------------------------------------------------
    def handle_active_label_changed(self, label_id):
        if label_id < 0:
            return
        self._active_label = label_id
        self.active_label_changed.emit(self._active_label)

    #-------------------------------------------------------------------------------------------------------
    def handle_smart_paint_changed(self, state):
        self._smart_paint = True if state == Qt.CheckState.Checked else False
        self.smart_paint_changed.emit(self._smart_paint)
        
    #-------------------------------------------------------------------------------------------------------
    def handle_fix_non_active_labels_changed(self, state):
        self._fix_non_active_labels = True if state == Qt.CheckState.Checked else False
        self.fix_non_active_labels_changed.emit(self._fix_non_active_labels)

    #-------------------------------------------------------------------------------------------------------
    def handle_hu_lo_range_muscle_changed(self, value):
        self._hu_lo_muscle = value
        self.hu_lo_muscle_changed.emit(self._hu_lo_muscle)

    #-------------------------------------------------------------------------------------------------------
    def handle_hu_hi_range_muscle_changed(self, value):
        self._hu_hi_muscle = value
        self.hu_hi_muscle_changed.emit(self._hu_hi_muscle)

    #-------------------------------------------------------------------------------------------------------
    def handle_hu_lo_range_vat_changed(self, value):
        self._hu_lo_vat = value
        self.hu_lo_vat_changed.emit(self._hu_lo_vat)

    #-------------------------------------------------------------------------------------------------------
    def handle_hu_hi_range_vat_changed(self, value):
        self._hu_hi_vat = value
        self.hu_hi_vat_changed.emit(self._hu_hi_vat)

    #-------------------------------------------------------------------------------------------------------
    def handle_hu_lo_range_sat_changed(self, value):
        self._hu_lo_sat = value
        self.hu_lo_sat_changed.emit(self._hu_lo_sat)

    #-------------------------------------------------------------------------------------------------------
    def handle_hu_hi_range_sat_changed(self, value):
        self._hu_hi_sat = value
        self.hu_hi_sat_changed.emit(self._hu_hi_sat)

    #-------------------------------------------------------------------------------------------------------
    def handle_overall_opacity_changed(self, value):
        opacity = float(value) / 100.0
        self._overall_opacity_label.setText(str(opacity))
        self._overall_opacity = opacity
        self.overall_opacity_changed.emit(opacity)

    #-------------------------------------------------------------------------------------------------------
    def handle_reset_zoom_button(self):
        self.reset_zoom.emit()

    #-------------------------------------------------------------------------------------------------------
    def handle_clear_all_labels_button(self):
        self.clear_labels.emit()


import slicer

#################################################################################################################
# Load this script in 3D Slicer as follows:
# - Open Python console: View > Python Console
# - Copy paste following line and run it: 
#
#   exec(open("D:\\SoftwareDevelopment\\GitHub\\mosamatic2\\mosamatic2\\src\\experiments\\liversegments\\main.py").read())
#
#################################################################################################################

# Define segment names and their corresponding file paths
segment_names = ["S1", "S2", "S3", "S4", "S5", "S6", "S7", "S8", "VESSELS"]
file_paths = [
    "D:\\Mosamatic\\TestData\\totalsegmentatortask\\liver_segment_1.nii.gz",
    "D:\\Mosamatic\\TestData\\totalsegmentatortask\\liver_segment_2.nii.gz",
    "D:\\Mosamatic\\TestData\\totalsegmentatortask\\liver_segment_3.nii.gz",
    "D:\\Mosamatic\\TestData\\totalsegmentatortask\\liver_segment_4.nii.gz",
    "D:\\Mosamatic\\TestData\\totalsegmentatortask\\liver_segment_5.nii.gz",
    "D:\\Mosamatic\\TestData\\totalsegmentatortask\\liver_segment_6.nii.gz",
    "D:\\Mosamatic\\TestData\\totalsegmentatortask\\liver_segment_7.nii.gz",
    "D:\\Mosamatic\\TestData\\totalsegmentatortask\\liver_segment_8.nii.gz",
    "D:\\Mosamatic\\TestData\\totalsegmentatortask\\liver_vessels.nii.gz",
]

# Define colors as RGB triples (values between 0 and 1)
colors = {
    "S1": (1.0, 0.0, 0.0),     # Red
    "S2": (0.0, 1.0, 0.0),     # Green
    "S3": (0.0, 0.0, 1.0),     # Blue
    "S4": (1.0, 1.0, 0.0),     # Yellow
    "S5": (1.0, 0.0, 1.0),     # Magenta
    "S6": (0.0, 1.0, 1.0),     # Cyan
    "S7": (0.6, 0.4, 0.2),     # Brown
    "S8": (0.5, 0.5, 0.5),     # Gray
    "VESSELS": (0.0, 0.0, 0.0) # Vessels
}

slicer.mrmlScene.Clear()

ctVolume = slicer.util.loadVolume("G:\\My Drive\\data\\Mosamatic\\testdata\\output\\boadockerpipeline\\dicom2niftitask\\patient1.nii.gz")
# vrLogic = slicer.modules.volumerendering.logic()
# vrNode = vrLogic.CreateDefaultVolumeRenderingNodes(ctVolume)

# # Adjust rendering preset (e.g. CT-AAA, CT-Bone, etc.)
# displayNode = vrNode.GetDisplayNode()
# displayNode.SetVisibility(True)  # Show volume rendering
# vrLogic.CopyDisplayToVolumeRenderingDisplayNode(vrLogic.GetPresetByName("CT-AAA"), displayNode)

layoutManager = slicer.app.layoutManager()
layoutManager.setLayout(slicer.vtkMRMLLayoutNode.SlicerLayoutFourUpView)  # show 2D + 3D

for comp in ["Red", "Yellow", "Green"]:
    sliceWidget = layoutManager.sliceWidget(comp)
    sliceWidget.sliceController().setSliceVisible(True)

# Create a new segmentation node
segmentationNode = slicer.mrmlScene.AddNewNodeByClass("vtkMRMLSegmentationNode", "LiverSegments")

# Import each segment mask and assign it a name and color
for name, path in zip(segment_names, file_paths):
    labelmapNode = slicer.util.loadLabelVolume(path)
    slicer.modules.segmentations.logic().ImportLabelmapToSegmentationNode(labelmapNode, segmentationNode)
    segment = segmentationNode.GetSegmentation().GetNthSegment(segmentationNode.GetSegmentation().GetNumberOfSegments() - 1)
    segment.SetName(name)
    if name in colors:
        segment.SetColor(*colors[name])
    slicer.mrmlScene.RemoveNode(labelmapNode)  # optional cleanup

# Create 3D surface for all segments
segmentationNode.CreateDefaultDisplayNodes()
segmentationNode.GetSegmentation().CreateRepresentation('Closed surface')
segmentationNode.GetDisplayNode().SetVisibility3D(True)
segmentationNode.GetDisplayNode().SetOpacity3D(0.2)



from PySide6.QtWidgets import (
    QLineEdit,
    QHBoxLayout,
    QVBoxLayout,
    QFormLayout,
    QPushButton,
    QFileDialog,
    QLabel,
    QSlider,
)
from PySide6.QtCore import Qt
from mosamatic2.ui.widgets.panels.visualizations.visualization import Visualization
from mosamatic2.ui.widgets.panels.visualizations.musclefatsegmentationuncertaintyvisualization.musclefatsegmentationuncertaintyviewer import MuscleFatSegmentationUncertaintyViewer
from mosamatic2.core.managers.logmanager import LogManager
from mosamatic2.ui.settings import Settings
from mosamatic2.ui.utils import is_macos

LOG = LogManager()
PANEL_TITLE = 'MuscleFatSegmentationUncertaintyVisualization'
PANEL_NAME = 'musclefatsegmentationuncertaintyvisualization'


class MuscleFatSegmentationVisualization(Visualization):
    def __init__(self):
        super(MuscleFatSegmentationVisualization, self).__init__()
        self.set_title(PANEL_TITLE)
        self._images_line_edit = None
        self._image_file_select_button = None
        self._segmentation_line_edit = None
        self._segmentation_file_select_button = None
        self._load_data_button = None
        self._lo_slider_value_label = None
        self._lo_slider = None
        self._hi_slider_value_label = None
        self._hi_slider = None
        self._muscle_fat_segmentation_uncertainty_viewer = None
        self._form_layout = None
        self._settings = None
        self.init_layout()

    def image_line_edit(self):
        if not self._images_line_edit:
            self._images_line_edit = QLineEdit(self.settings().get(f'{PANEL_NAME}/image'))
        return self._images_line_edit
    
    def image_file_select_button(self):
        if not self._image_file_select_button:
            self._image_file_select_button = QPushButton('Select file')
            self._image_file_select_button.clicked.connect(self.handle_image_file_select_button)
        return self._image_file_select_button
    
    def segmentation_line_edit(self):
        if not self._segmentation_line_edit:
            self._segmentation_line_edit = QLineEdit(self.settings().get(f'{PANEL_NAME}/segmentation'))
        return self._segmentation_line_edit
    
    def segmentation_file_select_button(self):
        if not self._segmentation_file_select_button:
            self._segmentation_file_select_button = QPushButton('Select file')
            self._segmentation_file_select_button.clicked.connect(self.handle_segmentation_file_select_button)
        return self._segmentation_file_select_button
    
    def load_data_button(self):
        if not self._load_data_button:
            self._load_data_button = QPushButton('Load')
            self._load_data_button.clicked.connect(self.handle_load_data_button)
        return self._load_data_button
    
    def lo_slider_value_label(self):
        if not self._lo_slider_value_label:
            self._lo_slider_value_label = QLabel('')
        return self._lo_slider_value_label
    
    def lo_slider(self):
        if not self._lo_slider:
            self._lo_slider = QSlider(Qt.Horizontal)
            self._lo_slider.setRange(-29, 150)
            self._lo_slider.setValue(30)
            self._lo_slider.valueChanged.connect(self.handle_lo_slider)
            self.lo_slider_value_label().setText(str(30))
        return self._lo_slider
    
    def hi_slider_value_label(self):
        if not self._hi_slider_value_label:
            self._hi_slider_value_label = QLabel('')
        return self._hi_slider_value_label
    
    def hi_slider(self):
        if not self._hi_slider:
            self._hi_slider = QSlider(Qt.Horizontal)
            self._hi_slider.setRange(-29, 150)
            self._hi_slider.setValue(150)
            self._hi_slider.valueChanged.connect(self.handle_hi_slider)
            self.hi_slider_value_label().setText(str(150))
        return self._hi_slider
    
    def muscle_fat_segmentation_uncertainty_viewer(self):
        if not self._muscle_fat_segmentation_uncertainty_viewer:
            self._muscle_fat_segmentation_uncertainty_viewer = MuscleFatSegmentationUncertaintyViewer()
        return self._muscle_fat_segmentation_uncertainty_viewer
    
    def form_layout(self):
        if not self._form_layout:
            self._form_layout = QFormLayout()
            if is_macos():
                self._form_layout.setFieldGrowthPolicy(QFormLayout.ExpandingFieldsGrow)
        return self._form_layout

    def settings(self):
        if not self._settings:
            self._settings = Settings()
        return self._settings

    def init_layout(self):
        image_layout = QHBoxLayout()
        image_layout.addWidget(self.image_line_edit())
        image_layout.addWidget(self.image_file_select_button())
        segmentation_layout = QHBoxLayout()
        segmentation_layout.addWidget(self.segmentation_line_edit())
        segmentation_layout.addWidget(self.segmentation_file_select_button())
        lo_slider_layout = QHBoxLayout()
        lo_slider_layout.addWidget(self.lo_slider_value_label())
        lo_slider_layout.addWidget(self.lo_slider())
        hi_slider_layout = QHBoxLayout()
        hi_slider_layout.addWidget(self.hi_slider_value_label())
        hi_slider_layout.addWidget(self.hi_slider())
        self.form_layout().addRow('Image file', image_layout)
        self.form_layout().addRow('Segmentation file (probabilities)', segmentation_layout)
        self.form_layout().addRow('', lo_slider_layout)
        self.form_layout().addRow('', hi_slider_layout)
        layout = QVBoxLayout()
        layout.addLayout(self.form_layout())
        layout.addWidget(self.load_data_button())
        layout.addWidget(self.muscle_fat_segmentation_uncertainty_viewer().navigation_toolbar())
        layout.addWidget(self.muscle_fat_segmentation_uncertainty_viewer())
        self.setLayout(layout)
        self.setObjectName(PANEL_NAME)

    def handle_image_file_select_button(self):
        last_directory = self.settings().get('last_directory')
        file_path, _ = QFileDialog.getOpenFileName(dir=last_directory)
        if file_path:
            self.image_line_edit().setText(file_path)
            self.settings().set('last_directory', file_path)

    def handle_segmentation_file_select_button(self):
        last_directory = self.settings().get('last_directory')
        file_path, _ = QFileDialog.getOpenFileName(dir=last_directory)
        if file_path:
            self.segmentation_line_edit().setText(file_path)
            self.settings().set('last_directory', file_path)

    def handle_load_data_button(self):
        self.muscle_fat_segmentation_uncertainty_viewer().load_data(
            self.image_line_edit().text(),
            self.segmentation_line_edit().text(),
            lo_hu=self.lo_slider().value(),
            hi_hu=self.hi_slider().value(),
        )

    def handle_lo_slider(self, value):
        self.muscle_fat_segmentation_uncertainty_viewer().update_lo_hu(value)
        self.lo_slider_value_label().setText(str(value))

    def handle_hi_slider(self, value):
        self.muscle_fat_segmentation_uncertainty_viewer().update_hi_hu(value)
        self.hi_slider_value_label().setText(str(value))

    def save_inputs_and_parameters(self):
        self.settings().set(f'{PANEL_NAME}/image', self.image_line_edit().text())
        self.settings().set(f'{PANEL_NAME}/segmentation', self.segmentation_line_edit().text())


import os
import shutil
import tempfile
from totalsegmentator.python_api import totalsegmentator
from mosamatic2.core.tasks.task import Task
from mosamatic2.core.managers.logmanager import LogManager
from mosamatic2.core.utils import is_gpu_available

LOG = LogManager()
TOTAL_SEGMENTATOR_OUTPUT_DIR = os.path.join(tempfile.gettempdir(), 'total_segmentator_output')


class TotalSegmentatorTask(Task):
    INPUTS = ['scans']
    PARAMS = ['tasks', 'format']

    def __init__(self, inputs, params, output, overwrite):
        super(TotalSegmentatorTask, self).__init__(inputs, params, output, overwrite)
        LOG.info(f'Using temporary output directory: {TOTAL_SEGMENTATOR_OUTPUT_DIR}')
        LOG.info(f'GPU available: {is_gpu_available()}')

    def load_scan_dirs(self):
        scan_dirs = []
        for d in os.listdir(self.input('scans')):
            scan_dir = os.path.join(self.input('scans'), d)
            if os.path.isdir(scan_dir):
                scan_dirs.append(scan_dir)
        return scan_dirs
    
    def load_scans(self):
        scans = []
        for f in os.listdir(self.input('scans')):
            if f.endswith('.nii.gz'):
                scan = os.path.join(self.input('scans'), f)
                if os.path.isfile(scan):
                    scans.append(scan)
        return scans
    
    def extract_masks(self, scan_dir_or_file):
        os.makedirs(TOTAL_SEGMENTATOR_OUTPUT_DIR, exist_ok=True)
        tasks = self.param('tasks').split(",") if self.param('tasks') else []
        for task in tasks:
            LOG.info(f'Running task {task}...')
            totalsegmentator(input=scan_dir_or_file, output=TOTAL_SEGMENTATOR_OUTPUT_DIR, task=task)

    def delete_total_segmentator_output(self):
        if os.path.exists(TOTAL_SEGMENTATOR_OUTPUT_DIR):
            shutil.rmtree(TOTAL_SEGMENTATOR_OUTPUT_DIR)

    def run(self):
        if self.param('format') == 'dicom':
            scan_dirs_or_files = self.load_scan_dirs()
        elif self.param('format') == 'nifti':
            scan_dirs_or_files = self.load_scans()
        else:
            LOG.error('Unknown format: {}'.format(self.param('format')))
            return
        nr_steps = len(scan_dirs_or_files)
        for step in range(nr_steps):
            scan_dir_or_file = scan_dirs_or_files[step]
            scan_dir_or_file_name = os.path.split(scan_dir_or_file)[1]
            if self.param('format') == 'nifti':
                scan_dir_or_file_name = scan_dir_or_file_name[:-7]
            try:
                self.extract_masks(scan_dir_or_file)
            except Exception as e:
                LOG.error(f'{scan_dir_or_file}: Could not extract masks [{str(e)}]. Skipping scan...')
            self.set_progress(step, nr_steps)
            LOG.info(f'Copying temporary output to final output directory...')
            for f in os.listdir(TOTAL_SEGMENTATOR_OUTPUT_DIR):
                if f.endswith('.nii') or f.endswith('.nii.gz'):
                    f_path = os.path.join(TOTAL_SEGMENTATOR_OUTPUT_DIR, f)
                    target_file = os.path.join(self.output(), f'{scan_dir_or_file_name}_{f}')
                    shutil.copyfile(f_path, target_file)
                    LOG.info(f'Copied {f} to {target_file}')
            LOG.info('Cleaning up Total Segmentator temporary output...')
            self.delete_total_segmentator_output()


import os
import shutil
import zipfile
import utils


LOCAL_ZIP_DIR = 'M:\\data\\porsch\\original\\zip'
HOSPITALS = [
    "Amphia Ziekenhuis (18)",
    "AUMC", # (11)
    "Catharina Ziekenhuis (24)",
    "Erasmus MC (17)",
    "LUMC (28)",
    "Isala Ziekenhuis (21)",
    "Jeroen Bosch Ziekenhuis (27)",
    "Medisch Spectrum Twente (22)",
    "MUMC+ (26)", # ERROR (could not find CT scan corresponding to L3)
    "OLVG Amsterdam (15)",
    "Radboud UMC (23)",
    "St. Antonius (13)", # ERROR (could not find CT scan corresponding to L3)
    "Tjongerschans Heerenveen (19)",
    "UMC Utrecht (14)",
    "UMCG (20)",
]


def download_zip_file(client, remote_zip_file_path, hospital_name):
    remote_zip_file_name = os.path.split(remote_zip_file_path)[1]
    local_hospital_path = os.path.join(LOCAL_ZIP_DIR, hospital_name)
    os.makedirs(local_hospital_path, exist_ok=True)
    local_zip_file_path = os.path.join(local_hospital_path, remote_zip_file_name)
    print(f'Downloading {remote_zip_file_path} to {local_zip_file_path}...')
    client.download_sync(remote_path=remote_zip_file_path, local_path=local_zip_file_path)
    return local_zip_file_path


def count_nr_scans_to_process(file_mapping):
    count = 0
    for l3_file_path, _ in file_mapping.items():
        hospital_name = utils.get_parent_name(l3_file_path)
        if hospital_name in HOSPITALS:
            count += 1
    return count


def main():
    client = utils.get_webdav_client()
    file_mapping = utils.load_dictionary('filemapping.json')
    print(f'Found {count_nr_scans_to_process(file_mapping)} scans to process')
    start_time = utils.current_time_in_seconds()
    for l3_file_path, remote_zip_file_path in file_mapping.items():
        local_hospital_name = utils.get_parent_name(l3_file_path)
        if local_hospital_name in HOSPITALS:
            start_time_inner = utils.current_time_in_seconds()
            download_zip_file(client, remote_zip_file_path, local_hospital_name)
            print(f'Elapsed time: {utils.duration(utils.elapsed_time_in_seconds(start_time_inner))}')
    print(f'Total elapsed time: {utils.duration(utils.elapsed_time_in_seconds(start_time))}')


if __name__ == '__main__':
    main()


import os
import shutil
from mosamatic2.core.utils import is_dicom, load_dicom

DATA_DIR = "D:\\Mosamatic\\NicoleHildebrand\\15-09-2025\\UMCG_scans_NDH"
DATA_DIR_PREP = "D:\\Mosamatic\\NicoleHildebrand\\15-09-2025\\UMCG_scans_NDH_prep"


for d in os.listdir(DATA_DIR):
    if d.startswith('HPB_'):
        print(f'Processing {d}...')
        source_dir = os.path.join(DATA_DIR, d) # e.g. HPB_1
        # Find one or more series per patients
        series_list = {}
        for root, dirs, files in os.walk(source_dir):
            for f in files:
                f_path = os.path.join(root, f)
                if is_dicom(f_path):
                    p = load_dicom(f_path, stop_before_pixels=True)
                    if p:
                        if 'SeriesInstanceUID' in p:
                            if p.SeriesInstanceUID not in series_list.keys():
                                series_list[p.SeriesInstanceUID] = []
                            series_list[p.SeriesInstanceUID].append(f_path)
                        else:
                            print(f'Missing SeriesInstanceUID in DICOM {f_path}')
                    else:
                        print(f'Could not load DICOM {f_path} or missing SeriesInstanceUID')
        # Copy each series to its own target directory
        count = 1
        for k, v in series_list.items():
            target_dir = os.path.join(DATA_DIR_PREP, f'{d}_{count}')
            os.makedirs(target_dir, exist_ok=True)
            for f in v:
                shutil.copy(f, target_dir)
                print(f)
            count += 1


import pydicom
import numpy as np
from matplotlib.backends.backend_qtagg import FigureCanvasQTAgg as FigureCanvas
from matplotlib.backends.backend_qtagg import NavigationToolbar2QT as NavigationToolbar
from matplotlib.figure import Figure


class MuscleFatSegmentationUncertaintyViewer(FigureCanvas):
    def __init__(self, parent=None, width=14, height=7, dpi=100):
        self._figure = Figure(figsize=(width, height), dpi=dpi)
        self._axes = self._figure.subplots(1, 2)
        super(MuscleFatSegmentationUncertaintyViewer, self).__init__(self._figure)
        self.setParent(parent)
        self._image = None
        self._segmentation = None
        self._muscle_mask = None
        self._lo_hu = 30
        self._hi_hu = 150
        self._toolbar = NavigationToolbar(self, parent)

    def figure(self):
        return self._figure
    
    def axes(self):
        return self._axes
    
    def navigation_toolbar(self):
        return self._toolbar
    
    # HELPERS

    def load_data(self, image_file, segmentation_file, lo_hu, hi_hu):
        self._lo_hu = lo_hu
        self._hi_hu = hi_hu
        # Load image
        p = pydicom.dcmread(image_file)
        self._image = p.pixel_array.astype(np.float32)
        self._image = self._image * float(getattr(p, "RescaleSlope", 1.0)) + float(getattr(p, "RescaleIntercept", 0.0))
        # Load segmentation mask
        self._segmentation = np.load(segmentation_file).astype(np.uint8)
        # Initialize muscle masks
        self._muscle_mask = (self._segmentation == 1)
        overlay_muscle = self.rgba_from_binary_mask(self._muscle_mask, rgba=(1.0, 0.0, 0.0), alpha=1)
        overlay_muscle_thresholded = self.rgba_from_binary_mask_thresholded(
            self._image, self._muscle_mask, label=1, hu_lo=self._lo_hu, hu_hi=self._hi_hu, alpha=1)
        # Plot the data
        self.axes()[0].imshow(self._image, cmap='gray')
        self.axes()[0].imshow(overlay_muscle)
        self.axes()[0].axis("off")
        self.axes()[0].set_title('Muscle (red)')
        self.axes()[1].imshow(self._image, cmap="gray")
        self.axes()[1].imshow(overlay_muscle_thresholded)
        self.axes()[1].axis("off")
        self.axes()[1].set_title(f"Muscle low-RA (yellow) and high-RA (red)")
        self._figure.tight_layout()
        self.draw()

    def update_lo_hu(self, value):
        self._lo_hu = value
        self.axes()[1].clear()
        self.axes()[1].imshow(self._image, cmap="gray")
        self.axes()[1].imshow(self.rgba_from_binary_mask_thresholded(
            self._image, self._muscle_mask, label=1, hu_lo=self._lo_hu, hu_hi=self._hi_hu, alpha=1))
        self.axes()[1].axis("off")
        self.axes()[1].set_title(f"Muscle low-RA (yellow) and high-RA (red)")
        self.draw_idle()

    def update_hi_hu(self, value):
        self._hi_hu = value
        self.axes()[1].clear()
        self.axes()[1].imshow(self._image, cmap="gray")
        self.axes()[1].imshow(self.rgba_from_binary_mask_thresholded(
            self._image, self._muscle_mask, label=1, hu_lo=self._lo_hu, hu_hi=self._hi_hu, alpha=1))
        self.axes()[1].axis("off")
        self.axes()[1].set_title(f"Muscle low-RA (yellow) and high-RA (red)")
        self.draw_idle()

    def rgba_from_binary_mask(self, mask, rgba, alpha):
        out = np.zeros((*mask.shape, 4), dtype=np.float32)
        out[mask] = (rgba[0], rgba[1], rgba[2], alpha)
        return out

    def rgba_from_binary_mask_thresholded(self, image, mask, label, hu_lo, hu_hi, alpha):
        mask = (mask == label)
        red = mask & (image >= hu_lo) & (image <= hu_hi)
        yellow = mask & (image < hu_lo)
        overlay = np.zeros((*image.shape, 4), dtype=np.float32)
        overlay[red] = (1.0, 0.0, 0.0, alpha)
        overlay[yellow] = (1.0, 1.0, 0.0, alpha)
        return overlay



import utils

file_mapping = utils.load_dictionary('filemapping.json')
time_per_scan = 30/42.0
counts = {}
for l3_file_path, _ in file_mapping.items():
    hospital_name = utils.get_parent_name(l3_file_path)
    if hospital_name not in counts.keys():
        counts[hospital_name] = [0, 0]
    counts[hospital_name][0] += 1
    counts[hospital_name][1] += time_per_scan
utils.save_dictionary(counts, 'countsandtimings.json')


from totalsegmentator.python_api import totalsegmentator

def main():
    totalsegmentator(input="D:\\Mosamatic\\TotalSegmentator\\patient1.nii.gz", output="D:\\Mosamatic\\TotalSegmentator\\output", fast=False)


if __name__ == '__main__':
    main()


# %%
import os
import pathlib

import matplotlib.pyplot as plt
import numpy as np
import torch

from models import UNet
# from models2 import AttentionUNet
from param_loader import Params


def normalize(data: np.ndarray, lower_bound: int, upper_bound: int) -> np.ndarray:
    """Normalize input: 'data' between 'lower_bound' and 'upper_bound',
    and scale between [0, 1].

    Parameters
    ----------
    data : np.ndarray
        Input image
    lower_bound : int
        Minimum Hounsfield Unit
    upper_bound : int
        Maximum Hounsfield Unit

    Returns
    -------
    np.ndarray
        Normalized and scaled input
    """

    data = (data - lower_bound) / (upper_bound - lower_bound)
    data[data > 1] = 1
    data[data < 0] = 0
    return data


DEVICE = "cuda"

# Load params.json
param_path = pathlib.Path(os.getcwd()) / "params.json"

params = Params(param_path)

# Load Contour model
model_path = os.path.join(
    os.getcwd(),
    "assets/logs/20250325-122319contour/saved_models/highest_val.pt",
)

model_c = UNet(params, 2).to(device=DEVICE)
model_c.load_state_dict(torch.load(model_path, weights_only=False))
model_c.eval()

# Load example patient
patients = np.load(os.path.join(os.getcwd(), "img_foreign_object.npy"))
patients_labels = np.load(os.path.join(os.getcwd(), "labels_foreign_object.npy"))

image = patients[:, :, 6]
label = patients_labels[:, :, 6]

# Preprocess, normalization and bounds are now the same for contour and body composition
image = normalize(
    image,
    lower_bound=params.dict["lower_bound"],
    upper_bound=params.dict["upper_bound"],
)

# Create Contour mask
with torch.no_grad():
    # Create 4D Tensor input
    input = np.expand_dims(image, 0)
    input = np.expand_dims(input, 0)
    input = torch.Tensor(input)
    input = input.to(DEVICE, dtype=torch.float)

    # Predict
    prediction = model_c(input)
    prediction = torch.argmax(prediction, axis=1)
    prediction = prediction.squeeze()
    prediction = prediction.detach().cpu().numpy()

# Keep only torso
image *= prediction

# Remove contour model from memory if necessary
# del model

# Load body composition model
model_path = os.path.join(
    os.getcwd(),
    "assets/logs/20250401-144027-fixedMuscle/saved_models/highest_val.pt",
)
model = UNet(params, 4).to(device=DEVICE)
model.load_state_dict(torch.load(model_path, weights_only=False))
model.eval()

with torch.no_grad():
    input = np.expand_dims(image, 0)
    input = np.expand_dims(input, 0)
    input = torch.Tensor(input)

    input = input.to(DEVICE, dtype=torch.float)

    prediction = model(input)
    prediction = torch.argmax(prediction, axis=1)
    prediction = prediction.squeeze()
    prediction = prediction.detach().cpu().numpy()

plt.figure()
plt.imshow(image, cmap="gray")
plt.imshow(prediction, alpha=0.3)

plt.figure()
plt.imshow(label)
plt.title("Label")

plt.figure()
plt.imshow(prediction)
plt.title("Prediction")


# %%



#!/usr/bin/env python3
"""
VTK overlay demo: single-slice CT (from ONE DICOM file) + multi-label segmentation mask (NumPy)
with a color map (LUT) for:
  background=0 (transparent)
  muscle=1 (red)
  visceral fat=5 (yellow)
  subcutaneous fat=7 (cyan)

Run:
  python vtk_l3_overlay_dicom_multilabel.py --dicom slice.dcm --mask seg.npy
"""

import argparse
import sys
import numpy as np
import vtk
from vtkmodules.util import numpy_support
import pydicom


def load_ct_dicom_as_hu(dicom_path: str):
    ds = pydicom.dcmread(dicom_path)

    px = ds.pixel_array.astype(np.float32)

    slope = float(getattr(ds, "RescaleSlope", 1.0))
    intercept = float(getattr(ds, "RescaleIntercept", 0.0))
    ct_hu = px * slope + intercept

    # DICOM PixelSpacing = [row_spacing, col_spacing]
    if hasattr(ds, "PixelSpacing") and len(ds.PixelSpacing) >= 2:
        row_spacing = float(ds.PixelSpacing[0])
        col_spacing = float(ds.PixelSpacing[1])
        spacing_xy = (col_spacing, row_spacing)  # VTK order: x=cols, y=rows
    else:
        spacing_xy = (1.0, 1.0)

    return ct_hu.astype(np.int16), spacing_xy


def numpy2vtk_image(img2d: np.ndarray, spacing=(1.0, 1.0, 1.0), origin=(0.0, 0.0, 0.0)) -> vtk.vtkImageData:
    if img2d.ndim != 2:
        raise ValueError(f"Expected 2D array (H,W), got shape {img2d.shape}")

    h, w = img2d.shape
    vtk_img = vtk.vtkImageData()
    vtk_img.SetDimensions(w, h, 1)
    vtk_img.SetSpacing(float(spacing[0]), float(spacing[1]), float(spacing[2]))
    vtk_img.SetOrigin(float(origin[0]), float(origin[1]), float(origin[2]))

    flat = np.ascontiguousarray(img2d).ravel(order="C")
    vtk_arr = numpy_support.numpy_to_vtk(flat, deep=True)
    vtk_img.GetPointData().SetScalars(vtk_arr)
    return vtk_img


def build_segmentation_lut(alpha_default: float = 0.35) -> vtk.vtkLookupTable:
    max_label = 7
    lut = vtk.vtkLookupTable()
    lut.SetNumberOfTableValues(max_label + 1)
    lut.Build()

    # Default: transparent for all labels
    for i in range(max_label + 1):
        lut.SetTableValue(i, 0.0, 0.0, 0.0, 0.0)

    # Specific labels
    lut.SetTableValue(0, 0.0, 0.0, 0.0, 0.0)  # background transparent
    lut.SetTableValue(1, 1.0, 0.0, 0.0, float(alpha_default))  # muscle = red
    lut.SetTableValue(5, 1.0, 1.0, 0.0, float(alpha_default))  # visceral fat = yellow
    lut.SetTableValue(7, 0.0, 1.0, 1.0, float(alpha_default))  # subcutaneous fat = cyan

    return lut


def show_ct_with_multilabel_overlay(
    ct2d: np.ndarray,
    seg2d: np.ndarray,
    *,
    spacing_xy=(1.0, 1.0),
    window: float = 400.0,
    level: float = 40.0,
    overlay_alpha: float = 0.35,
    flipud: bool = False,
):
    if ct2d.shape != seg2d.shape:
        raise ValueError(f"CT shape {ct2d.shape} != seg shape {seg2d.shape}")

    if flipud:
        ct2d = np.flipud(ct2d)
        seg2d = np.flipud(seg2d)

    # Convert to VTK images
    ct_vtk = numpy2vtk_image(ct2d.astype(np.int16, copy=False), spacing=(spacing_xy[0], spacing_xy[1], 1.0))
    seg_vtk = numpy2vtk_image(seg2d.astype(np.uint8, copy=False), spacing=(spacing_xy[0], spacing_xy[1], 1.0))

    # CT: window/level -> RGB
    wl = vtk.vtkImageMapToWindowLevelColors()
    wl.SetInputData(ct_vtk)
    wl.SetWindow(float(window))
    wl.SetLevel(float(level))
    wl.Update()

    ct_actor = vtk.vtkImageActor()
    ct_actor.GetMapper().SetInputConnection(wl.GetOutputPort())

    # Segmentation: LUT -> RGBA
    lut = build_segmentation_lut(alpha_default=overlay_alpha)

    seg_rgba = vtk.vtkImageMapToColors()
    seg_rgba.SetInputData(seg_vtk)
    seg_rgba.SetLookupTable(lut)
    seg_rgba.SetOutputFormatToRGBA()
    seg_rgba.Update()

    seg_actor = vtk.vtkImageActor()
    seg_actor.GetMapper().SetInputConnection(seg_rgba.GetOutputPort())

    # Render
    ren = vtk.vtkRenderer()
    ren.AddActor(ct_actor)
    ren.AddActor(seg_actor)
    ren.ResetCamera()

    rw = vtk.vtkRenderWindow()
    rw.AddRenderer(ren)
    rw.SetSize(900, 900)

    iren = vtk.vtkRenderWindowInteractor()
    iren.SetRenderWindow(rw)
    iren.SetInteractorStyle(vtk.vtkInteractorStyleImage())

    rw.Render()
    iren.Start()


def main(argv=None):
    # p = argparse.ArgumentParser(description="VTK overlay: CT from single DICOM + mask from NumPy .npy")
    # p.add_argument("--dicom", required=True, help="Path to single CT DICOM slice file")
    # p.add_argument("--mask", required=True, help="Path to mask .npy (H,W)")
    # p.add_argument("--window", type=float, default=400.0)
    # p.add_argument("--level", type=float, default=40.0)
    # p.add_argument("--alpha", type=float, default=0.35)
    # p.add_argument("--flipud", action="store_true", help="Flip arrays vertically before display")
    # args = p.parse_args(argv)

    # ct_hu, spacing_xy = load_ct_dicom_as_hu(args.dicom)
    ct_hu, spacing_xy = load_ct_dicom_as_hu("D:\\Mosamatic\\MartijnBroen\\02-10-2025_twee_plaatjes\\org\\419474.dcm")
    # mask = np.load(args.mask)
    mask = np.load("D:\\Mosamatic\\MartijnBroen\\02-10-2025_twee_plaatjes\\defaultpipeline\\segmentmusclefatl3tensorflowtask\\419474.dcm.seg.npy")

    if mask.ndim != 2:
        raise ValueError(f"Mask must be 2D (H,W). Got shape {mask.shape}")

    # Sanity check sizes
    if mask.shape != ct_hu.shape:
        raise ValueError(
            f"Mask shape {mask.shape} does not match DICOM pixel array shape {ct_hu.shape}. "
            "Make sure the mask corresponds to this slice and has same (rows, cols)."
        )
    
    uniq = np.unique(mask)
    print(f"Unique labels in segmentation: {uniq.tolist()}")

    show_ct_with_multilabel_overlay(
        ct_hu,
        mask,
        spacing_xy=spacing_xy,
        window=400,
        level=50,
        overlay_alpha=1,
        flipud=True,
    )    


if __name__ == "__main__":
    main(sys.argv[1:])





import os
import dicom2nifti
from mosamatic2.core.tasks.task import Task
from mosamatic2.core.managers.logmanager import LogManager
from mosamatic2.core.utils import is_dicom

LOG = LogManager()


class Dicom2NiftiTask(Task):
    """
    This task converts DICOM scans to NIFTI. Each scan should be stored in a separate 
    directory (scan_dir). This task does not convert a single DICOM image!!
    """
    INPUTS = ['scans']
    PARAMS = ['compressed']

    def __init__(self, inputs, params, output, overwrite):
        super(Dicom2NiftiTask, self).__init__(inputs, params, output, overwrite)

    def load_scan_dirs(self):
        scan_dirs = []
        for d in os.listdir(self.input('scans')):
            scan_dir = os.path.join(self.input('scans'), d)
            if os.path.isdir(scan_dir):
                scan_dirs.append(scan_dir)
        return scan_dirs

    def run(self):
        scan_dirs = self.load_scan_dirs()
        nr_steps = len(scan_dirs)
        for step in range(nr_steps):
            scan_dir = scan_dirs[step]
            scan_name = os.path.split(scan_dir)[1]
            if self.param('compressed'):
                nifti_file_name = scan_name + '.nii.gz'
            else:
                nifti_file_name = scan_name + '.nii'
            LOG.info(f'Converting DICOM series in {scan_dir} to {nifti_file_name}')
            dicom2nifti.dicom_series_to_nifti(
                scan_dir,
                os.path.join(self.output(), nifti_file_name),
                reorient_nifti=True,
            )
            self.set_progress(step, nr_steps)


import os
import shutil
import pydicom
import pydicom.errors
import pandas as pd

ORIGINAL_DIR = "D:\\Mosamatic\\L1-L5\\Original"
ORIGINAL_COLLECTED_DIR = "D:\\Mosamatic\\L1-L5\\OriginalCollected"
SCORES_FILE_PATH = "D:\\Mosamatic\\L1-L5\\Out\\defaultpipeline\\calculatescorestask\\bc_scores.csv"


def is_dicom(f_path):
    try:
        pydicom.dcmread(f_path, stop_before_pixels=True)
        return True
    except pydicom.errors.InvalidDicomError:
        return False
    

def collect_files():
    for d in os.listdir(ORIGINAL_DIR):
        if d.startswith('Patient '):
            d_path = os.path.join(ORIGINAL_DIR, d)
            for f in os.listdir(d_path):
                f_path = os.path.join(d_path, f)
                if os.path.isfile(f_path):
                    if is_dicom(f_path):
                        f_name = os.path.split(f_path)[1]
                        d_name = d.replace(' ', '_')
                        target_f_name = f'{d_name}_{f_name}'
                        target_f_path = os.path.join(ORIGINAL_COLLECTED_DIR, target_f_name)
                        shutil.copy(f_path, target_f_path)
                        print(target_f_path)


def main():
    collect_files()


if __name__ == '__main__':
    main()


import os
import shutil
import numpy as np
import pydicom
import SimpleITK as sitk
import matplotlib.pyplot as plt
from scipy.ndimage import zoom
from mosamatic2.core.utils import load_dicom

DICOM_DIR = 'D:\\Mosamatic\\NicoleHildebrand\\24-04-2025_coronale_CT\\MUMC004_3_coronaal'
DICOM_DIR_RENAMED = 'D:\\Mosamatic\\NicoleHildebrand\\24-04-2025_coronale_CT\\MUMC004_3_coronaal_renamed'
DICOM_DIR_DECOMPRESSED = 'D:\\Mosamatic\\NicoleHildebrand\\24-04-2025_coronale_CT\\MUMC004_3_coronaal_decompressed'
NIFTI_OUTPUT_FILE = 'D:\\Mosamatic\\NicoleHildebrand\\24-04-2025_coronale_CT\\MUMC004_3_coronal_to_axial.nii.gz'
L3_INDEX = 167


def load_dicom_series():
    os.makedirs(DICOM_DIR_RENAMED, exist_ok=True)
    os.makedirs(DICOM_DIR_DECOMPRESSED, exist_ok=True)
    for f in os.listdir(DICOM_DIR):
        f_path = os.path.join(DICOM_DIR, f)
        shutil.copyfile(f_path, os.path.join(DICOM_DIR_RENAMED, f[1:]))
    for f in os.listdir(DICOM_DIR_RENAMED):
        f_path = os.path.join(DICOM_DIR_RENAMED, f)
        p = load_dicom(f_path)
        p.decompress()
        p.save_as(os.path.join(DICOM_DIR_DECOMPRESSED, f))
    reader = sitk.ImageSeriesReader()
    series_IDs = reader.GetGDCMSeriesIDs(DICOM_DIR_DECOMPRESSED)
    if not series_IDs:
        raise RuntimeError(f"No DICOM series found in {DICOM_DIR_DECOMPRESSED}")
    series_file_names = reader.GetGDCMSeriesFileNames(DICOM_DIR_DECOMPRESSED, series_IDs[0])
    reader.SetFileNames(series_file_names)
    img = reader.Execute()
    return img


def resample_isotropic(img, new_spacing=(1.0, 1.0, 1.0)):
    orig_spacing = img.GetSpacing()
    orig_size = img.GetSize()
    new_size = [
        int(round(orig_size[i] * (orig_spacing[i] / new_spacing[i])))
        for i in range(3)
    ]
    resampled = sitk.Resample(
        img,
        new_size,
        sitk.Transform(),
        sitk.sitkLinear,
        img.GetOrigin(),
        new_spacing,
        img.GetDirection(),
        0,
        img.GetPixelID()
    )
    return resampled


def reformat_coronal_to_axial(img):
    arr = sitk.GetArrayFromImage(img)   # shape: (z, y, x)
    axial_arr = np.transpose(arr, (1, 0, 2))  # (z,y,x) -> (y,z,x)
    axial_img = sitk.GetImageFromArray(axial_arr)
    axial_img.SetSpacing(img.GetSpacing())  # keep spacing isotropic
    return axial_img


def apply_hu_scaling(img, dicom_dir):
    sample_file = next(f for f in os.listdir(dicom_dir) if f.endswith(".dcm"))
    ds = pydicom.dcmread(os.path.join(dicom_dir, sample_file))
    slope = getattr(ds, "RescaleSlope", 1.0)
    intercept = getattr(ds, "RescaleIntercept", 0.0)    
    arr = sitk.GetArrayFromImage(img).astype(np.float32)
    hu_arr = arr * slope + intercept
    hu_img = sitk.GetImageFromArray(hu_arr)
    hu_img.CopyInformation(img)
    return hu_img

def rescale_image(pixel_array, target_size=512, hu_air=-1000):
    rows, cols = pixel_array.shape
    # Pad to square with air
    new_dim = max(rows, cols)
    padded = np.full((new_dim, new_dim), hu_air, dtype=np.float32)
    padded[:rows, :cols] = pixel_array
    # Resize to target size with cubic interpolation
    zoom_factor = target_size / new_dim
    resized = zoom(padded, zoom=zoom_factor, order=3)
    print(f'resized: ({np.min(resized)}, {np.max(resized)})')
    return resized

# def rescale_image(pixel_array, rows=238, columns=470, target_size=512):
#     rescale_slope = 1
#     rescale_intercept = -1024
#     hu_array = pixel_array * rescale_slope + rescale_intercept
#     hu_air = -1000
#     new_rows = max(rows, columns)
#     new_cols = max(rows, columns)
#     padded_hu_array = np.full((new_rows, new_cols), hu_air, dtype=hu_array.dtype)
#     padded_hu_array[:pixel_array.shape[0], :pixel_array.shape[1]] = hu_array
#     pixel_array_padded = (padded_hu_array - rescale_intercept) / rescale_slope
#     pixel_array_padded = pixel_array_padded.astype(pixel_array.dtype) # Image now has largest dimensions
#     pixel_array_rescaled = zoom(pixel_array_padded, zoom=(target_size / new_rows), order=3) # Cubic interpolation
#     pixel_array_rescaled = pixel_array_rescaled.astype(pixel_array.dtype)
#     print(f'pixel_array_rescaled: ({np.min(pixel_array_rescaled)}, {np.max(pixel_array_rescaled)})')
#     return pixel_array_rescaled

def preview_slices(img, num_slices=6):
    arr = sitk.GetArrayFromImage(img)  # z,y,x
    z_slices = arr.shape[0]
    l3_slice = arr[L3_INDEX, :, :]
    l3_slice = rescale_image(l3_slice)
    fig, axes = plt.subplots(1, 1, figsize=(15, 5))
    axes.imshow(l3_slice, cmap="gray", vmin=-200, vmax=300)
    axes.axis("off")
    # step = max(1, z_slices // num_slices)
    # indices = list(range(0, z_slices, step))[:num_slices]
    # fig, axes = plt.subplots(1, len(indices), figsize=(15, 5))
    # for ax, idx in zip(axes, indices):
    #     ax.imshow(arr[idx, :, :], cmap="gray", vmin=-200, vmax=300)
    #     ax.set_title(f"Slice {idx}")
    #     ax.axis("off")
    plt.tight_layout()
    plt.show()


def main():
    img = load_dicom_series()
    resampled = resample_isotropic(img, new_spacing=(1.0, 1.0, 1.0))
    axial = reformat_coronal_to_axial(resampled)
    # axial = apply_hu_scaling(axial, DICOM_DIR_DECOMPRESSED)
    sitk.WriteImage(axial, NIFTI_OUTPUT_FILE)
    preview_slices(axial, num_slices=6)


if __name__ == '__main__':
    main()


import os
import sys
import vtk
from PySide6.QtWidgets import QApplication, QMainWindow, QVBoxLayout, QWidget
from vtkmodules.qt.QVTKRenderWindowInteractor import QVTKRenderWindowInteractor

DICOM_DIR = 'G:\\My Drive\\data\\Mosamatic\\testdata\\CT\\patient1'


class MultiStyleInteractor(vtk.vtkInteractorStyle):
    def __init__(self, slice_actors, image_data, ren_3d):
        super().__init__()
        self.slice_actors = slice_actors
        self.image_data = image_data
        self.ren_3d = ren_3d

        # delegate style for 3D
        self.trackball = vtk.vtkInteractorStyleTrackballCamera()

        # hook events manually
        self.AddObserver("MouseWheelForwardEvent", self.OnMouseWheelForward)
        self.AddObserver("MouseWheelBackwardEvent", self.OnMouseWheelBackward)
        self.AddObserver("LeftButtonPressEvent", self.OnLeftButtonDown)
        self.AddObserver("LeftButtonReleaseEvent", self.OnLeftButtonUp)

    def _get_renderer_at_position(self, x, y):
        rw = self.GetInteractor().GetRenderWindow()
        size = rw.GetSize()
        nx, ny = x / size[0], y / size[1]
        collection = rw.GetRenderers()
        collection.InitTraversal()
        for i in range(collection.GetNumberOfItems()):
            ren = collection.GetNextItem()
            xmin, ymin, xmax, ymax = ren.GetViewport()
            if xmin <= nx <= xmax and ymin <= ny <= ymax:
                return ren
        return None

    def _scroll_slice(self, renderer, step):
        entry = self.slice_actors[renderer]
        orientation = entry["orientation"]
        idx = entry["index"]
        dims = self.image_data.GetDimensions()

        if orientation == "axial":
            idx = min(max(idx + step, 0), dims[2] - 1)
            entry["actor"].SetDisplayExtent(0, dims[0]-1, 0, dims[1]-1, idx, idx)
        elif orientation == "sagittal":
            idx = min(max(idx + step, 0), dims[0] - 1)
            entry["actor"].SetDisplayExtent(idx, idx, 0, dims[1]-1, 0, dims[2]-1)
        elif orientation == "coronal":
            idx = min(max(idx + step, 0), dims[1] - 1)
            entry["actor"].SetDisplayExtent(0, dims[0]-1, idx, idx, 0, dims[2]-1)

        entry["index"] = idx
        self.GetInteractor().GetRenderWindow().Render()

    # --- Events ---
    def OnMouseWheelForward(self, obj=None, ev=None):
        x, y = self.GetInteractor().GetEventPosition()
        renderer = self._get_renderer_at_position(x, y)
        if renderer in self.slice_actors:
            self._scroll_slice(renderer, +1)
        elif renderer == self.ren_3d:
            self.trackball.SetInteractor(self.GetInteractor())
            self.trackball.OnMouseWheelForward()

    def OnMouseWheelBackward(self, obj=None, ev=None):
        x, y = self.GetInteractor().GetEventPosition()
        renderer = self._get_renderer_at_position(x, y)
        if renderer in self.slice_actors:
            self._scroll_slice(renderer, -1)
        elif renderer == self.ren_3d:
            self.trackball.SetInteractor(self.GetInteractor())
            self.trackball.OnMouseWheelBackward()

    def OnLeftButtonDown(self, obj=None, ev=None):
        x, y = self.GetInteractor().GetEventPosition()
        renderer = self._get_renderer_at_position(x, y)
        if renderer == self.ren_3d:
            self.trackball.SetInteractor(self.GetInteractor())
            self.trackball.OnLeftButtonDown()

    def OnLeftButtonUp(self, obj=None, ev=None):
        x, y = self.GetInteractor().GetEventPosition()
        renderer = self._get_renderer_at_position(x, y)
        if renderer == self.ren_3d:
            self.trackball.SetInteractor(self.GetInteractor())
            self.trackball.OnLeftButtonUp()


def create_slice_renderer(image_data, orientation, slice_index, viewport):
    dims = image_data.GetDimensions()

    slice_actor = vtk.vtkImageActor()
    mapper = slice_actor.GetMapper()
    mapper.SetInputData(image_data)

    if orientation == "axial":
        slice_actor.SetDisplayExtent(0, dims[0]-1, 0, dims[1]-1, slice_index, slice_index)
    elif orientation == "sagittal":
        slice_actor.SetDisplayExtent(slice_index, slice_index, 0, dims[1]-1, 0, dims[2]-1)
    elif orientation == "coronal":
        slice_actor.SetDisplayExtent(0, dims[0]-1, slice_index, slice_index, 0, dims[2]-1)

    ren = vtk.vtkRenderer()
    ren.AddActor(slice_actor)
    ren.SetViewport(*viewport)

    # Fix camera orientation
    cam = ren.GetActiveCamera()
    if orientation == "axial":
        cam.SetFocalPoint(dims[0]//2, dims[1]//2, slice_index)
        cam.SetPosition(dims[0]//2, dims[1]//2, slice_index + 500)
        cam.SetViewUp(0, -1, 0)
    elif orientation == "sagittal":
        cam.SetFocalPoint(slice_index, dims[1]//2, dims[2]//2)
        cam.SetPosition(slice_index + 500, dims[1]//2, dims[2]//2)
        cam.SetViewUp(0, 0, 1)
    elif orientation == "coronal":
        cam.SetFocalPoint(dims[0]//2, slice_index, dims[2]//2)
        cam.SetPosition(dims[0]//2, slice_index + 500, dims[2]//2)
        cam.SetViewUp(0, 0, 1)

    cam.ParallelProjectionOn()
    ren.ResetCamera()

    return ren, slice_actor


def create_volume_renderer(image_data, viewport):
    volume_mapper = vtk.vtkGPUVolumeRayCastMapper()
    volume_mapper.SetInputData(image_data)

    color = vtk.vtkColorTransferFunction()
    color.AddRGBPoint(0, 0.0, 0.0, 0.0)
    color.AddRGBPoint(500, 1.0, 0.5, 0.3)
    color.AddRGBPoint(1000, 1.0, 1.0, 0.9)

    opacity = vtk.vtkPiecewiseFunction()
    opacity.AddPoint(0, 0.0)
    opacity.AddPoint(500, 0.2)
    opacity.AddPoint(1000, 0.8)

    volume_property = vtk.vtkVolumeProperty()
    volume_property.SetColor(color)
    volume_property.SetScalarOpacity(opacity)
    volume_property.ShadeOn()
    volume_property.SetInterpolationTypeToLinear()

    volume = vtk.vtkVolume()
    volume.SetMapper(volume_mapper)
    volume.SetProperty(volume_property)

    ren = vtk.vtkRenderer()
    ren.AddVolume(volume)
    ren.SetViewport(*viewport)
    ren.ResetCamera()
    return ren


class MainWindow(QMainWindow):
    def __init__(self, image_data, parent=None):
        super().__init__(parent)
        self.setWindowTitle("VTK Slice + 3D Viewer with Scrolling - PySide6")

        frame = QWidget()
        layout = QVBoxLayout()
        frame.setLayout(layout)
        self.setCentralWidget(frame)

        vtk_widget = QVTKRenderWindowInteractor(frame)
        layout.addWidget(vtk_widget)

        render_window = vtk_widget.GetRenderWindow()
        interactor = render_window.GetInteractor()

        dims = image_data.GetDimensions()
        axial_index = dims[2] // 2
        sagittal_index = dims[0] // 2
        coronal_index = dims[1] // 2

        ren_axial, actor_axial = create_slice_renderer(image_data, "axial", axial_index, (0.0, 0.5, 0.5, 1.0))
        ren_sagittal, actor_sagittal = create_slice_renderer(image_data, "sagittal", sagittal_index, (0.5, 0.5, 1.0, 1.0))
        ren_coronal, actor_coronal = create_slice_renderer(image_data, "coronal", coronal_index, (0.0, 0.0, 0.5, 0.5))
        ren_3d = create_volume_renderer(image_data, (0.5, 0.0, 1.0, 0.5))

        render_window.AddRenderer(ren_axial)
        render_window.AddRenderer(ren_sagittal)
        render_window.AddRenderer(ren_coronal)
        render_window.AddRenderer(ren_3d)

        slice_actors = {
            ren_axial: {"actor": actor_axial, "orientation": "axial", "index": axial_index},
            ren_sagittal: {"actor": actor_sagittal, "orientation": "sagittal", "index": sagittal_index},
            ren_coronal: {"actor": actor_coronal, "orientation": "coronal", "index": coronal_index},
        }

        style = MultiStyleInteractor(slice_actors, image_data, ren_3d)
        interactor.SetInteractorStyle(style)
        interactor.Initialize()


if __name__ == "__main__":
    app = QApplication(sys.argv)
    reader = vtk.vtkDICOMImageReader()
    reader.SetDirectoryName(DICOM_DIR)
    reader.Update()
    image_data = reader.GetOutput()
    window = MainWindow(image_data)
    window.show()
    sys.exit(app.exec())



#!/usr/bin/env python3
"""
Show a CT slice (single DICOM) with an overlay computed from HU values *within the muscle region*:

- Red:    muscle pixels with 30 <= HU <= 50
- Yellow: muscle pixels with HU < 30
- Transparent elsewhere

Optionally also create (and save) a new labeled mask:
  0 = not muscle / background
  1 = muscle in-range (red)
  2 = muscle low (yellow)

Run:
  python hu_based_muscle_overlay.py --dicom slice.dcm --seg seg.npy
  python hu_based_muscle_overlay.py --dicom slice.dcm --seg seg.npy --save-mask new_mask.npy
"""

import argparse
import numpy as np
import matplotlib.pyplot as plt
import pydicom


def load_ct_hu(dicom_path: str) -> np.ndarray:
    ds = pydicom.dcmread(dicom_path)
    img = ds.pixel_array.astype(np.float32)

    slope = float(getattr(ds, "RescaleSlope", 1.0))
    intercept = float(getattr(ds, "RescaleIntercept", 0.0))
    hu = img * slope + intercept
    return hu


def window_image(hu: np.ndarray, window: float, level: float) -> np.ndarray:
    lo = level - window / 2.0
    hi = level + window / 2.0
    return np.clip(hu, lo, hi)


def build_hu_overlay_within_muscle(
    hu: np.ndarray,
    seg: np.ndarray,
    *,
    muscle_label: int = 1,
    hu_low_thresh: float = 30.0,
    hu_high_thresh: float = 50.0,
    alpha: float = 0.35,
):
    """
    Returns:
      overlay_rgba: (H,W,4) float32 in [0..1]
      new_mask:     (H,W) uint8 labels:
                     0 = not muscle/background
                     1 = muscle in [hu_low_thresh, hu_high_thresh]
                     2 = muscle < hu_low_thresh
    """
    if hu.shape != seg.shape:
        raise ValueError(f"Shape mismatch: HU {hu.shape} vs seg {seg.shape}")

    muscle = (seg == muscle_label)

    red = muscle & (hu >= hu_low_thresh) & (hu <= hu_high_thresh)
    yellow = muscle & (hu < hu_low_thresh)

    overlay = np.zeros((*hu.shape, 4), dtype=np.float32)
    overlay[red] = (1.0, 0.0, 0.0, alpha)       # red
    overlay[yellow] = (1.0, 1.0, 0.0, alpha)    # yellow

    new_mask = np.zeros(hu.shape, dtype=np.uint8)
    new_mask[red] = 1
    new_mask[yellow] = 2

    return overlay, new_mask


def overlay_from_binary(mask: np.ndarray, rgba, alpha: float) -> np.ndarray:
    """mask (H,W) bool -> overlay (H,W,4)"""
    out = np.zeros((*mask.shape, 4), dtype=np.float32)
    out[mask] = (rgba[0], rgba[1], rgba[2], alpha)
    return out


def overlay_hu_within_muscle(
    hu: np.ndarray,
    muscle_mask: np.ndarray,
    hu_low: float,
    hu_high: float,
    alpha: float,
) -> np.ndarray:
    """
    Returns RGBA overlay:
      - red for hu in [hu_low, hu_high]
      - yellow for hu < hu_low
      - transparent elsewhere
    """
    red = muscle_mask & (hu >= hu_low) & (hu <= hu_high)
    yellow = muscle_mask & (hu < hu_low)
    overlay = np.zeros((*hu.shape, 4), dtype=np.float32)
    overlay[red] = (1.0, 0.0, 0.0, alpha)
    overlay[yellow] = (1.0, 1.0, 0.0, alpha)
    return overlay


def main():
    # ap = argparse.ArgumentParser()
    # ap.add_argument("--dicom", required=True, help="Path to single CT DICOM slice")
    # ap.add_argument("--seg", required=True, help="Path to segmentation .npy (H,W); muscle label assumed 1")
    # ap.add_argument("--muscle-label", type=int, default=1, help="Label value for muscle in seg mask")
    # ap.add_argument("--window", type=float, default=400.0)
    # ap.add_argument("--level", type=float, default=40.0)
    # ap.add_argument("--alpha", type=float, default=0.35)
    # ap.add_argument("--hu-low", type=float, default=30.0, help="Low HU threshold (yellow below this)")
    # ap.add_argument("--hu-high", type=float, default=50.0, help="High HU threshold (red up to this)")
    # ap.add_argument("--flipud", action="store_true", help="Flip HU+seg vertically before display")
    # ap.add_argument("--save-mask", default=None, help="If set, save new labeled mask (.npy)")
    # args = ap.parse_args()

    hu = load_ct_hu("D:\\Mosamatic\\MartijnBroen\\02-10-2025_twee_plaatjes\\org\\419474.dcm")
    seg = np.load("D:\\Mosamatic\\MartijnBroen\\02-10-2025_twee_plaatjes\\defaultpipeline\\segmentmusclefatl3tensorflowtask\\419474.dcm.seg.npy")

    if seg.ndim != 2 or hu.ndim != 2:
        raise ValueError(f"Expected 2D arrays. Got hu.ndim={hu.ndim}, seg.ndim={seg.ndim}")
    if seg.shape != hu.shape:
        raise ValueError(f"Shape mismatch: CT {hu.shape} vs seg {seg.shape}")

    print("Unique labels in seg:", np.unique(seg).tolist())

    flipud = False

    if flipud:
        hu = np.flipud(hu)
        seg = np.flipud(seg)

    # overlay, new_mask = build_hu_overlay_within_muscle(
    #     hu,
    #     seg,
    #     muscle_label=1,
    #     hu_low_thresh=30,
    #     hu_high_thresh=60,
    #     alpha=1,
    # )

    # if save_mask:
    #     np.save(save_mask, new_mask)
    #     print(f"Saved new mask to: {args.save_mask}")
    #     print("New mask labels present:", np.unique(new_mask).tolist())

    # Display
    hu_w = window_image(hu, 400, 50)

    muscle_mask = (seg == 1)

    # Plot 1 overlay: pure muscle (red)
    overlay_muscle = overlay_from_binary(muscle_mask, rgba=(1.0, 0.0, 0.0), alpha=1)

    # Plot 2 overlay: HU-thresholded within muscle (red/yellow)
    overlay_hu = overlay_hu_within_muscle(
        hu=hu,
        muscle_mask=muscle_mask,
        hu_low=30,
        hu_high=100,
        alpha=1,
    )

    # plt.figure(figsize=(8, 8))
    # plt.imshow(hu_w, cmap="gray")
    # plt.imshow(overlay)  # RGBA overlay computed from HU thresholds within muscle
    # plt.axis("off")
    # plt.title(f"Muscle HU overlay: red={30}..{60}, yellow< {30}")
    # plt.show()

    fig, axes = plt.subplots(1, 2, figsize=(14, 7))

    axes[0].imshow(hu_w, cmap="gray")
    axes[0].imshow(overlay_muscle)
    axes[0].axis("off")
    axes[0].set_title("Muscle mask (red)")

    axes[1].imshow(hu_w, cmap="gray")
    axes[1].imshow(overlay_hu)
    axes[1].axis("off")
    axes[1].set_title(f"Muscle HU overlay: red={30}..{60}, yellow< {30}")

    plt.tight_layout()
    plt.show()


if __name__ == "__main__":
    main()



import os
import shutil
import pydicom
import pydicom.errors


DATA_DIR = 'M:\\data\\contrastsensitivity\\lieke\\original'
DATA_DIR_PREPARED = 'M:\\data\\contrastsensitivity\\lieke\\orirginal_prepared'
PHASES = [
    'unenhanced', 
    'arterial', 
    'venous',
]


def print_phases(d_path):
    phases = []
    for d in os.listdir(d_path):
        phases.append(d)
    print(', '.join(phases))


def is_dicom(f_path):
    try:
        pydicom.dcmread(f_path, stop_before_pixels=True)
        return True
    except pydicom.errors.InvalidDicomError:
        return False


def get_file(d_path):
    for root, dirs, files in os.walk(d_path):
        for f in files:
            if f != 'DICOMDIR':
                f_path = os.path.join(root, f)
                if is_dicom(f_path):
                    return f_path
    return None


def main():
    for d in os.listdir(DATA_DIR):
        d_path = os.path.join(DATA_DIR, d)
        if os.path.isdir(d_path):
            for phase_dir in os.listdir(d_path):
                if phase_dir in PHASES:
                    phase_dir_path = os.path.join(d_path, phase_dir)
                    if os.path.isdir(phase_dir_path):
                        f_path = get_file(phase_dir_path)
                        if f_path:
                            f_name = f'{d}_{phase_dir}.dcm'
                            shutil.copy(f_path, os.path.join(DATA_DIR_PREPARED, f_name))


if __name__ == '__main__':
    main()


import os
import numpy as np
import nibabel as nib
import matplotlib.pyplot as plt
from mosamatic2.core.tasks.task import Task
from mosamatic2.core.managers.logmanager import LogManager
from mosamatic2.core.utils import (
    convert_numpy_array_to_png_image,
    AlbertaColorMap, 
)
LOG = LogManager()


class SegmentationNifti2NumpyTask(Task):
    INPUTS = ['segmentations']
    PARAMS = ['png']

    def __init__(self, inputs, params, output, overwrite):
        super(SegmentationNifti2NumpyTask, self).__init__(inputs, params, output, overwrite)

    def load_segmentations(self):
        segmentations = []
        for f in os.listdir(self.input('segmentations')):
            if f.endswith('.seg.npy.nii.gz'):
                f_path = os.path.join(self.input('segmentations'), f)
                segmentations.append(f_path)
                print(f_path)
        return segmentations
    
    def load_segmentation_as_nifti(self, segmentation):
        nifti = nib.load(segmentation)
        narray = np.asanyarray(nifti.dataobj)
        narray = narray[..., 0]
        narray = np.rot90(narray, k=3, axes=(0, 1))
        return narray
    
    def create_png_from_array(self, data, file_path):
        png_file_name = os.path.split(file_path)[1] + '.png'
        convert_numpy_array_to_png_image(
            data, 
            self.output(),
            AlbertaColorMap(), 
            png_file_name,
            fig_width=10, fig_height=10,
        )

    def run(self):
        segmentations = self.load_segmentations()        
        nr_steps = len(segmentations)
        for step in range(nr_steps):
            segmentation = segmentations[step]
            segmentation_narray = self.load_segmentation_as_nifti(segmentation)
            segmentation_narray_name = os.path.split(segmentation)[1][:-7]
            segmentation_narray_path = os.path.join(self.output(), segmentation_narray_name)
            np.save(segmentation_narray_path, segmentation_narray)
            if self.param('png'):
                self.create_png_from_array(segmentation_narray, segmentation_narray_path)
            self.set_progress(step, nr_steps)


import os
import shutil

DATA_DIR_PREP = "D:\\Mosamatic\\NicoleHildebrand\\15-09-2025\\UMCG\\UMCG_scans_NDH_prep"
DATA_DIR_DONE = "D:\\Mosamatic\\NicoleHildebrand\\15-09-2025\\UMCG\\UMCG_scans_NDH_prep_done"
L3_DIR_OK = "D:\\Mosamatic\\NicoleHildebrand\\15-09-2025\\UMCG\\output\\selectslicefromscanstask"

for f in os.listdir(L3_DIR_OK):
    scan_dir_name = os.path.splitext(f)[0]
    scan_dir_name = scan_dir_name[3:]
    source_dir = os.path.join(DATA_DIR_PREP, scan_dir_name)
    if os.path.isdir(source_dir):
        shutil.move(source_dir, DATA_DIR_DONE)
        print(f'Moved scan {scan_dir_name} to {DATA_DIR_DONE}')